# sage.doctest: needs sage.symbolic
"""
Some tests about Gosper sums.

References
==========

- [PWZ1997]_

TESTS::

    sage: _ = var('a b k m n r')
    sage: SR(1).gosper_sum(k)
    k
    sage: SR(1).gosper_sum(k,0,n)
    n + 1
    sage: SR(1).gosper_sum(k, a, b)
    -a + b + 1
    sage: a.gosper_sum(k)
    a*k
    sage: a.gosper_sum(k,0,n)
    a*(n + 1)
    sage: a.gosper_sum(k,a,b)
    -(a - b - 1)*a
    sage: k.gosper_sum(k)
    1/2*(k - 1)*k
    sage: k.gosper_sum(k,0,n)
    1/2*(n + 1)*n
    sage: k.gosper_sum(k, a, b)
    -1/2*(a + b)*(a - b - 1)
    sage: k.gosper_sum(k, a+1, b)
    -1/2*(a + b + 1)*(a - b)
    sage: k.gosper_sum(k, a, b+1)
    -1/2*(a + b + 1)*(a - b - 2)
    sage: (k^3).gosper_sum(k)
    1/4*(k - 1)^2*k^2
    sage: (k^3).gosper_sum(k, a, b)
    -1/4*(a^2 + b^2 - a + b)*(a + b)*(a - b - 1)

    sage: (1/k).gosper_sum(k)
    Traceback (most recent call last):
    ...
    ValueError: expression not Gosper-summable
    sage: x = (1/k/(k+1)/(k+2)/(k+3)/(k+5)/(k+7)).gosper_sum(k,a,b)
    sage: y = sum(1/k/(k+1)/(k+2)/(k+3)/(k+5)/(k+7) for k in range(5,500))
    sage: assert x.subs(a==5,b==499) == y

The following are from A==B, p.78 ff. Since the resulting expressions
get more and more complicated with many correct representations that
may differ depending on future capabilities, we check correctness by
doing random summations::

    sage: def check(ex, var, val1, val2from, val2to):
    ....:     import random
    ....:     symb = SR.var('symb')
    ....:     s1 = ex.gosper_sum(var, val1, symb)
    ....:     R = random.SystemRandom
    ....:     val2 = R().randint(val2from, val2to)
    ....:     s2 = sum(ex.subs(var==i) for i in range(val1, val2+1))
    ....:     assert s1.subs(symb==val2) == s2

    sage: def check_unsolvable(ex, *args):
    ....:     try:
    ....:         SR(ex).gosper_sum(*args)
    ....:         raise AssertionError
    ....:     except ValueError:
    ....:         pass

    sage: check((4*n+1) * factorial(n)/factorial(2*n+1), n, 0, 10, 20)
    sage: check_unsolvable(factorial(k), k,0,n)
    sage: (k * factorial(k)).gosper_sum(k,1,n)
    n*factorial(n) + factorial(n) - 1
    sage: (k * factorial(k)).gosper_sum(k)
    factorial(k)
    sage: check_unsolvable(binomial(n,k), k,0,n)
    sage: ((-1)^k*binomial(n,k)).gosper_sum(k,0,n)
    0
    sage: ((-1)^k*binomial(n,k)).gosper_sum(k,0,a)
    -(-1)^a*(a - n)*binomial(n, a)/n
    sage: (binomial(1/2,a-k+1)*binomial(1/2,a+k)).gosper_sum(k,1,b)
    (2*a + 2*b - 1)*(a - b + 1)*b*binomial(1/2, a + b)*binomial(1/2, a - b + 1)/((2*a + 1)*a)
    sage: t = (binomial(2*k,k)/4^k).gosper_sum(k,0,n); t
    (2*n + 1)*binomial(2*n, n)/4^n
    sage: t = t.gosper_sum(n,0,n); t
    1/3*(2*n + 3)*(2*n + 1)*binomial(2*n, n)/4^n
    sage: t = t.gosper_sum(n,0,n); t
    1/15*(2*n + 5)*(2*n + 3)*(2*n + 1)*binomial(2*n, n)/4^n
    sage: t = t.gosper_sum(n,0,n); t
    1/105*(2*n + 7)*(2*n + 5)*(2*n + 3)*(2*n + 1)*binomial(2*n, n)/4^n
    sage: (binomial(2*k+2*a,2*a)*binomial(2*k,k)/binomial(k+a,a)/4^k).gosper_sum(k,0,n)
    (2*a + 2*n + 1)*binomial(2*a + 2*n, 2*a)*binomial(2*n, n)/(4^n*(2*a + 1)*binomial(a + n, a))
    sage: (4^k/binomial(2*k,k)).gosper_sum(k,0,n)
    1/3*(2*4^n*n + 2*4^n + binomial(2*n, n))/binomial(2*n, n)

    # The following are from A==B, 5.7 Exercises
    sage: for k in range(1,5): (n^k).gosper_sum(n,0,m)
    1/2*(m + 1)*m
    1/6*(2*m + 1)*(m + 1)*m
    1/4*(m + 1)^2*m^2
    1/30*(3*m^2 + 3*m - 1)*(2*m + 1)*(m + 1)*m
    sage: for k in range(1,4): (n^k*2^n).gosper_sum(n,0,m)
    2*2^m*m - 2*2^m + 2
    2*2^m*m^2 - 4*2^m*m + 6*2^m - 6
    2*2^m*m^3 - 6*2^m*m^2 + 18*2^m*m - 26*2^m + 26
    sage: (1 / (n^2 + sqrt(5)*n - 1)).gosper_sum(n,0,m) # known bug
    ....:                            # TODO: algebraic solutions
    sage: check((n^4 * 4^n / binomial(2*n, n)), n, 0, 10, 20)
    sage: check((factorial(3*n) / (factorial(n) * factorial(n+1) * factorial(n+2) * 27^n)), n,0,10,20)
    sage: (binomial(2*n, n)^2 / (n+1) / 4^(2*n)).gosper_sum(n,0,m)
    (2*m + 1)^2*binomial(2*m, m)^2/(4^(2*m)*(m + 1))
    sage: (((4*n-1) * binomial(2*n, n)^2) / (2*n-1)^2 / 4^(2*n)).gosper_sum(n,0,m)
    -binomial(2*m, m)^2/4^(2*m)
    sage: check(n * factorial(n-1/2)^2 / factorial(n+1)^2, n,0,10,20)

    sage: (n^2 * a^n).gosper_sum(n,0,m)
    (a^2*a^m*m^2 - 2*a*a^m*m^2 - 2*a*a^m*m + a^m*m^2 + a*a^m + 2*a^m*m + a^m - a - 1)*a/(a - 1)^3
    sage: ((n - r/2)*binomial(r, n)).gosper_sum(n,0,m)
    1/2*(m - r)*binomial(r, m)
    sage: x = var('x')
    sage: (factorial(n-1)^2 / factorial(n-x) / factorial(n+x)).gosper_sum(n,1,m)
    (m^2*factorial(m - 1)^2*factorial(x + 1)*factorial(-x + 1) + x^2*factorial(m + x)*factorial(m - x) - factorial(m + x)*factorial(m - x))/(x^2*factorial(m + x)*factorial(m - x)*factorial(x + 1)*factorial(-x + 1))
    sage: ((n*(n+a+b)*a^n*b^n)/factorial(n+a)/factorial(n+b)).gosper_sum(n,1,m).simplify_full()  # long time
    -(a^(m + 1)*b^(m + 1)*factorial(a - 1)*factorial(b - 1) - factorial(a + m)*factorial(b + m))/(factorial(a + m)*factorial(a - 1)*factorial(b + m)*factorial(b - 1))

    sage: check_unsolvable(1/n, n,1,m)
    sage: check_unsolvable(1/n^2, n,1,m)
    sage: check_unsolvable(1/n^3, n,1,m)
    sage: ((6*n + 3) / (4*n^4 + 8*n^3 + 8*n^2 + 4*n + 3)).gosper_sum(n,1,m)
    (m + 2)*m/(2*m^2 + 4*m + 3)
    sage: (2^n * (n^2 - 2*n - 1)/(n^2 * (n+1)^2)).gosper_sum(n,1,m)
    -2*(m^2 - 2^m + 2*m + 1)/(m + 1)^2
    sage: ((4^n * n^2)/((n+2) * (n+1))).gosper_sum(n,1,m)
    2/3*(2*4^m*m - 2*4^m + m + 2)/(m + 2)
    sage: check_unsolvable(2^n / (n+1), n,0,m-1)
    sage: check((4*(1-n) * (n^2-2*n-1) / n^2 / (n+1)^2 / (n-2)^2 / (n-3)^2), n, 4, 10, 20)
    sage: check(((n^4-14*n^2-24*n-9) * 2^n / n^2 / (n+1)^2 / (n+2)^2 / (n+3)^2), n, 1, 10, 20)

Exercises 3 (h), (i), (j) require symbolic product support so we leave
them out for now.

::

    sage: _ = var('a b k m n r')
    sage: check_unsolvable(binomial(2*n, n) * a^n, n)
    sage: (binomial(2*n,n)*(1/4)^n).gosper_sum(n)
    2*(1/4)^n*n*binomial(2*n, n)
    sage: ((k-1) / factorial(k)).gosper_sum(k)
    -k/factorial(k)
    sage: F(n, k) = binomial(n, k) / 2^n
    sage: check_unsolvable(F(n, k), k)
    sage: _ = (F(n+1,k)-F(n,k)).gosper_term(k)
    sage: F(n,k).WZ_certificate(n,k)
    1/2*k/(k - n - 1)
    sage: F(n, k) = binomial(n, k)^2 / binomial(2*n, n)
    sage: check_unsolvable(F(n, k), k)
    sage: _ =(F(n+1, k) - F(n, k)).gosper_term(k)
    sage: F(n,k).WZ_certificate(n,k)
    1/2*(2*k - 3*n - 3)*k^2/((k - n - 1)^2*(2*n + 1))
    sage: F(n, k) = binomial(n,k) * factorial(n) / factorial(k) / factorial(a-k) / factorial(a+n)
    sage: check_unsolvable(F(n, k), k)
    sage: _ = (F(n+1, k) - F(n, k)).gosper_term(k)
    sage: F(n,k).WZ_certificate(n,k)
    k^2/((a + n + 1)*(k - n - 1))

    sage: (1/n/(n+1)/(n+2)/(n+5)).gosper_sum(n)
    1/720*(55*n^5 + 550*n^4 + 1925*n^3 + 2510*n^2 - 1728)/((n + 4)*(n + 3)*(n + 2)*(n + 1)*n)
    sage: (1/n/(n+1)/(n+2)/(n+7)).gosper_sum(n)
    1/1050*(91*n^7 + 1911*n^6 + 15925*n^5 + 66535*n^4 + 142534*n^3 + 132104*n^2 - 54000)/((n + 6)*(n + 5)*(n + 4)*(n + 3)*(n + 2)*(n + 1)*n)
    sage: (1/n/(n+1)/(n+2)/(n+5)/(n+7)).gosper_sum(n)
    1/10080*(133*n^7 + 2793*n^6 + 23275*n^5 + 97755*n^4 + 213472*n^3 + 206892*n^2 - 103680)/((n + 6)*(n + 5)*(n + 4)*(n + 3)*(n + 2)*(n + 1)*n)

The following are from A=B, 7.2 WZ Proofs of the hypergeometric database::

    sage: _ = var('a b c i j k m n r')
    sage: F(n,k) = factorial(n+k)*factorial(b+k)*factorial(c-n-1)*factorial(c-b-1)/factorial(c+k)/factorial(n-1)/factorial(c-n-b-1)/factorial(k+1)/factorial(b-1)
    sage: F(n,k).WZ_certificate(n,k)
    -(c + k)*(k + 1)/((c - n - 1)*n)
    sage: F(n,k)=(-1)^(n+k)*factorial(2*n+c-1)*factorial(n)*factorial(n+c-1)/factorial(2*n+c-1-k)/factorial(2*n-k)/factorial(c+k-1)/factorial(k)
    sage: F(n,k).WZ_certificate(n,k)
    1/2*(c^2 - 2*c*k + k^2 + 7*c*n - 6*k*n + 10*n^2 + 4*c - 3*k + 10*n + 2)*(c + k - 1)*k/((c - k + 2*n + 1)*(c - k + 2*n)*(k - 2*n - 1)*(k - 2*n - 2))
    sage: F(n,k)=factorial(a+k-1)*factorial(b+k-1)*factorial(n)*factorial(n+c-b-a-k-1)*factorial(n+c-1)/factorial(k)/factorial(n-k)/factorial(k+c-1)/factorial(n+c-a-1)/factorial(n+c-b-1)
    sage: F(n,k).WZ_certificate(n,k)
    -(a + b - c + k - n)*(c + k - 1)*k/((a - c - n)*(b - c - n)*(k - n - 1))
    sage: F(n,k)=(-1)^k*binomial(n+b,n+k)*binomial(n+c,c+k)*binomial(b+c,b+k)/factorial(n+b+c)*factorial(n)*factorial(b)*factorial(c)
    sage: F(n,k).WZ_certificate(n,k)
    1/2*(b + k)*(c + k)/((b + c + n + 1)*(k - n - 1))
    sage: phi(t) = factorial(a+t-1)*factorial(b+t-1)/factorial(t)/factorial(-1/2+a+b+t)
    sage: psi(t) = factorial(t+a+b-1/2)*factorial(t)*factorial(t+2*a+2*b-1)/factorial(t+2*a-1)/factorial(t+a+b-1)/factorial(t+2*b-1)
    sage: F(n,k) = phi(k)*phi(n-k)*psi(n)
    sage: F(n,k).WZ_certificate(n,k)
    (2*a + 2*b + 2*k - 1)*(2*a + 2*b - 2*k + 3*n + 2)*(a - k + n)*(b - k + n)*k/((2*a + 2*b - 2*k + 2*n + 1)*(2*a + n)*(a + b + n)*(2*b + n)*(k - n - 1))

The following are also from A=B, 7 The WZ Phenomenon::

    sage: F(n,k) = factorial(n-i)*factorial(n-j)*factorial(i-1)*factorial(j-1)/factorial(n-1)/factorial(k-1)/factorial(n-i-j+k)/factorial(i-k)/factorial(j-k)
    sage: F(n,k).WZ_certificate(n,k)
    (k - 1)/n
    sage: F(n,k) = binomial(3*n,k)/8^n
    sage: F(n,k).WZ_certificate(n,k)
    1/8*(4*k^2 - 30*k*n + 63*n^2 - 22*k + 93*n + 32)*k/((k - 3*n - 1)*(k - 3*n - 2)*(k - 3*n - 3))

Example 7.5.1 gets a different but correct certificate (the certificate
in the book fails the proof)::

    sage: F(n,k) = 2^(k+1)*(k+1)*factorial(2*n-k-2)*factorial(n)/factorial(n-k-1)/factorial(2*n)
    sage: c = F(n,k).WZ_certificate(n,k); c
    -1/2*(k - 2*n + 1)*k/((k - n)*(2*n + 1))
    sage: G(n,k) = c*F(n,k)
    sage: t = (F(n+1,k) - F(n,k) - G(n,k+1) + G(n,k))
    sage: t.simplify_full().is_trivial_zero()  # long time
    True
    sage: c = k/2/(-1+k-n)
    sage: GG(n,k) = c*F(n,k)
    sage: t = (F(n+1,k) - F(n,k) - GG(n,k+1) + GG(n,k))
    sage: t.simplify_full().is_trivial_zero()  # long time
    False
"""



"""
This file collects tests requiring Cython.
"""
# ***************************************************************************
#       Copyright (C) 2012 Jeroen Demeyer <jdemeyer@cage.ugent.be>
#       Copyright (C) 2012 Simon King <simon.king@uni-jena.de>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ***************************************************************************

from sage.misc.fast_methods cimport FastHashable_class


cdef class ClassWithLargeHash(FastHashable_class):
    """
    This class tests against a bug with :class:`FastHashable_class`
    (in an earlier version of the patch at :issue:`11900`) that occurred
    on systems where ``sys.maxsize`` does not fit into a C int.

    TESTS::

        sage: import sage.tests.cython
        sage: C = sage.tests.cython.ClassWithLargeHash(); C
        A successfully created object with a very large hash
        sage: hash(C) == sys.maxsize
        True
    """
    def __init__(self):
        import sys
        self._hash = sys.maxsize

    def __repr__(self):
        return 'A successfully created object with a very large hash'



"""
This file (./sol/numbertheory_doctest.sage) was *autogenerated* from ./sol/numbertheory.tex,
with sagetex.sty version 2011/05/27 v2.3.1.
It contains the contents of all the sageexample environments from this file.
You should be able to doctest this file with:
sage -t ./sol/numbertheory_doctest.sage
It is always safe to delete this file; it is not used in typesetting your
document.

Sage example in ./sol/numbertheory.tex, line 9::

  sage: def enum_carmichael(N, verbose=True):
  ....:    p = 3; s = 0
  ....:    while p^3 <= N:
  ....:       s += enum_carmichael_p(N, p, verbose); p = next_prime(p)
  ....:    return s

Sage example in ./sol/numbertheory.tex, line 26::

  sage: def enum_carmichael_p (n, p, verbose):
  ....:    a = p; m = p*(p-1); q = p; s = 0
  ....:    while p*q^2 <= n:
  ....:       q = next_prime(q)
  ....:       s += enum_carmichael_pq(n, a, m, p, q, verbose)
  ....:    return s

Sage example in ./sol/numbertheory.tex, line 52::

  sage: def enum_carmichael_pq(n,a,m,p,q,verbose):
  ....:    if (a-q) % gcd(m,q*(q-1)) != 0: return 0
  ....:    s = 0
  ....:    a = crt (a, q, m, q*(q-1)); m = lcm(m,q*(q-1))
  ....:    while a <= p*q^2: a += m
  ....:    for t in range(a, n+1, m):
  ....:       r = t // (p*q)
  ....:       if is_prime(r) and t % (r-1) == 1:
  ....:          if verbose:
  ....:             print((p*q*r, factor(p*q*r)))
  ....:          s += 1
  ....:    return s

Sage example in ./sol/numbertheory.tex, line 68::

  sage: enum_carmichael(10^4)
  (561, 3 * 11 * 17)
  (1105, 5 * 13 * 17)
  (2465, 5 * 17 * 29)
  (1729, 7 * 13 * 19)
  (2821, 7 * 13 * 31)
  (8911, 7 * 19 * 67)
  (6601, 7 * 23 * 41)
  7
  sage: enum_carmichael(10^5, False)
  12
  sage: enum_carmichael(10^6, False)
  23
  sage: enum_carmichael(10^7, False)
  47

Sage example in ./sol/numbertheory.tex, line 94::

  sage: def aliq(n):
  ....:    l = [n]
  ....:    while n != 1:
  ....:       n = sigma(n) - n
  ....:       if n in l: break
  ....:       l.append(n)
  ....:    return l
  sage: l = aliq(840)  # long time
  sage: len(l), l[:5], l[-5:]  # long time
  (748, [840, 2040, 4440, 9240, 25320], [2714, 1606, 1058, 601, 1])

Sage example in ./sol/numbertheory.tex, line 106::

  sage: p = points([(i, log(l[i])/log(10)) for i in range(len(l))])  # long time

Sage example in ./sol/numbertheory.tex, line 173::

  sage: def rk_aux(xc, yc, d, r2):
  ....:     s = 0
  ....:     xmin = ceil((xc - sqrt(r2))/d)
  ....:     xmax = floor((xc + sqrt(r2))/d)
  ....:     for x in range(xmin,xmax+1):
  ....:         r3 = r2 - (d*x-xc)^2 # (d*y-yc)^2 <= r2 - (d*x-xc)^2
  ....:         ymin = ceil((yc - sqrt(r3))/d)
  ....:         ymax = floor((yc + sqrt(r3))/d)
  ....:         s += ymax + 1 - ymin
  ....:     return s

Sage example in ./sol/numbertheory.tex, line 219::

  sage: def rk(k): # returns (r_k^2, xc, yc)
  ....:     if k == 2: return 1/4, 1/2, 0
  ....:     dmax = (2*sqrt(k/pi)).n(); xamax = (sqrt(2*k/pi)).n()
  ....:     sol = (dmax/2)^2, 0, 0, 0
  ....:     for xa in range(0, floor(xamax)+1):
  ....:         # if xa=0, ya > 0 since A should differ from O
  ....:         yamin = max(xa, 1)
  ....:         for ya in range(yamin, floor(sqrt(dmax^2-xa^2))+1):
  ....:             xbmin = 0 # we want xb*ya <= xa^2+ya^2
  ....:             if xa == 0:
  ....:                 xbmin = 1 # O, A, B should not be aligned
  ....:             xbmax = min(floor(dmax), floor((xa*xa+ya*ya)/ya))
  ....:             for xb in range(xbmin, xbmax+1):
  ....:                 ybmax = floor(sqrt(dmax^2-xb^2))
  ....:                 if xa > 0: # we want xb*ya+yb*xa <= xa^2+ya^2
  ....:                     tmp = floor((xa*xa+ya*ya-xb*ya)/xa)
  ....:                     ybmax = min(ybmax, tmp)
  ....:                 # if xb=0, yb > 0 since B should differ from O
  ....:                 ybmin = 0
  ....:                 if xb == 0:
  ....:                     ybmin = 1
  ....:                 for yb in range(ybmin,ybmax+1):
  ....:                     d = 2*abs(xb*ya - xa*yb)
  ....:                     if d != 0:
  ....:                         ra2 = xa^2+ya^2; rb2 = xb^2+yb^2
  ....:                         xc = abs(ra2*yb - rb2*ya)
  ....:                         yc = abs(rb2*xa - ra2*xb)
  ....:                         r2 = ra2*rb2*((xa-xb)^2+(ya-yb)^2)
  ....:                         m = rk_aux(xc,yc,d,r2)
  ....:                         if m >= k and r2/d^2 < sol[0]:
  ....:                             sol = r2/d^2, xc/d, yc/d
  ....:     return sol

Sage example in ./sol/numbertheory.tex, line 253::

  sage: for k in range(2,10): print((k, rk(k)))
  (2, (1/4, 1/2, 0))
  (3, (1/2, 1/2, 1/2))
  (4, (1/2, 1/2, 1/2))
  (5, (1, 0, 1))
  (6, (5/4, 1/2, 1))
  (7, (25/16, 3/4, 1))
  (8, (2, 1, 1))
  (9, (2, 1, 1))

Sage example in ./sol/numbertheory.tex, line 283::

  sage: def plotrk(k):
  ....:    r2, x0, y0 = rk(k); r = n(sqrt(r2))
  ....:    var('x, y')
  ....:    c = implicit_plot((x-x0)^2+(y-y0)^2-r2,
  ....:        (x, x0-r-1/2, x0+r+1/2),(y, y0-r-1/2, y0+r+1/2))
  ....:    center = points([(x0,y0)], pointsize=50, color='black')
  ....:    # we want (i-x0)^2+(j-y0)^2 <= r2
  ....:    # thus |i-x0| <= r and |j-y0| <= r2 - (i-x0)^2
  ....:    l = [(i, j) for i in range(ceil(x0-r), floor(x0+r)+1)
  ....:                for j in range(ceil(y0-sqrt(r^2-(i-x0)^2)),
  ....:                               floor(y0+sqrt(r2-(i-x0)^2))+1)]
  ....:    d = points(l, pointsize=100)
  ....:    return (c+center+d).show(aspect_ratio=1, axes=True)

Sage example in ./sol/numbertheory.tex, line 377::

  sage: x1, x2, s2 = var('x1, x2, s2')
  sage: n1 = 9; C1 = integrate(x1^n1, x1, x2, s2); C1
  1/10*s2^10 - 1/10*x2^10

Sage example in ./sol/numbertheory.tex, line 390::

  sage: x3, s3 = var('x3, s3')
  sage: n2 = 7; C2 = integrate(C1.subs(s2=s3-x2)*x2^n2, x2, x3, s3/2); C2
  44923/229417943040*s3^18 - 1/80*s3^10*x3^8 + 1/9*s3^9*x3^9 - 9/20*s3^8*x3^10
  + 12/11*s3^7*x3^11 - 7/4*s3^6*x3^12 + 126/65*s3^5*x3^13 - 3/2*s3^4*x3^14
  + 4/5*s3^3*x3^15 - 9/32*s3^2*x3^16 + 1/17*s3*x3^17
"""



import types


def run_tests() -> None:
    """
    Run all memcheck tests
    """
    from sage.tests.memcheck import symbolic_expression
    run_tests_in_module(symbolic_expression)


def run_tests_in_module(mod: types.ModuleType) -> None:
    """
    Run all memcheck tests in the given module
    """
    for entry in dir(mod):
        if not entry.startswith('test_'):
            continue
        test_func = getattr(mod, entry)
        test_func()


if __name__ == '__main__':
    run_tests()



# sage.doctest: needs sage.combinat
r"""
Sage code for computing k-distant crossing numbers.

This code accompanies the article :arxiv:`0812.2725`. It is being
submitted because of a suggestion from
http://groups.google.com/group/sage-support/msg/3ea7ed2eeab0824a.

Right now, this code only computes k-dcrossings. If you are only
interested in the distribution, this is good enough because the extended
Kasraoui-Zeng involution tells us the distribution of k-dcrossings and
k-dnestings is symmetric. It would be nice, though, to have a function
which actually performed that involution.

AUTHORS:

- Dan Drake (2008-12-15): initial version.

EXAMPLES:

The example given in the paper. Note that in this format, we omit fixed
points since they cannot create any sort of crossing. ::

    sage: from sage.tests.arxiv_0812_2725 import *
    sage: dcrossing([(1,5), (2,4), (4,9), (6,12), (7,10), (10,11)])
    3
"""
#*****************************************************************************
# Copyright (C) 2008 Dan Drake <ddrake@member.ams.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or (at
# your option) any later version.
#
# See https://www.gnu.org/licenses/.
#*****************************************************************************
from sage.combinat.set_partition import SetPartitions as SetPartitions


def CompleteMatchings(n):
    """
    Return a generator for the complete matchings of the set [1..n].

    INPUT:

    - ``n`` -- nonnegative integer

    OUTPUT:

    A generator for the complete matchings of the set [1..n], or,
    what is basically the same thing, complete matchings of the
    graph K_n. Each complete matching is represented by a list of
    2-element tuples.

    EXAMPLES:

    There are 3 complete matchings on 4 vertices::

        sage: from sage.tests.arxiv_0812_2725 import *
        sage: [m for m in CompleteMatchings(4)]
        [[(3, 4), (1, 2)], [(2, 4), (1, 3)], [(2, 3), (1, 4)]]

    There are no complete matchings on an odd number of vertices; the
    number of complete matchings on an even number of vertices is a
    double factorial::

        sage: from sage.tests.arxiv_0812_2725 import *
        sage: [len([m for m in CompleteMatchings(n)]) for n in [0..8]]
        [1, 0, 1, 0, 3, 0, 15, 0, 105]

    The exact behavior of CompleteMatchings(n) if n is not a nonnegative
    integer depends on what [1..n] returns, and also on what range(1,
    len([1..n])) is.
    """
    yield from matchingsset(list(range(1, n + 1)))


def matchingsset(L):
    """
    Return a generator for complete matchings of the sequence L.

    This is not really meant to be called directly, but rather by
    CompleteMatchings().

    INPUT:

    - ``L`` -- a sequence. Lists, tuples, et cetera; anything that
      supports ``len()`` and slicing should work.

    OUTPUT:

    A generator for complete matchings on K_n, where n is the length
    of L and vertices are labeled by elements of L. Each matching is
    represented by a list of 2-element tuples.

    EXAMPLES::

        sage: from sage.tests.arxiv_0812_2725 import *
        sage: [m for m in matchingsset(('a', 'b', 'c', 'd'))]
        [[('c', 'd'), ('a', 'b')], [('b', 'd'), ('a', 'c')], [('b', 'c'), ('a', 'd')]]

    There's only one matching of the empty set/list/tuple: the empty
    matching. ::

        sage: [m for m in matchingsset(())]
        [[]]
    """
    if not L:
        yield []
    else:
        for k in range(1, len(L)):
            for m in matchingsset(L[1: k] + L[k + 1:]):
                yield m + [(L[0], L[k])]


def dcrossing(m_):
    """
    Return the largest k for which the given matching or set
    partition has a k-distant crossing.

    INPUT:

    - ``m`` -- a matching or set partition, as a list of 2-element tuples
      representing the edges. You'll need to call ``setp_to_edges()`` on
      the objects returned by ``SetPartitions()`` to put them into the
      proper format.

    OUTPUT:

    The largest k for which the object has a k-distant crossing.
    Matchings and set partitions with no crossings at all yield -1.

    EXAMPLES:

    The main example from the paper::

        sage: from sage.tests.arxiv_0812_2725 import *
        sage: dcrossing(setp_to_edges(Set(map(Set, [[1,5],[2,4,9],[3],[6,12],[7,10,11],[8]]))))
        3

    A matching example::

        sage: from sage.tests.arxiv_0812_2725 import *
        sage: dcrossing([(4, 7), (3, 6), (2, 5), (1, 8)])
        2

    TESTS:

    The empty matching and set partition are noncrossing::

        sage: dcrossing([])
        -1
        sage: dcrossing(Set([]))
        -1

    One edge::

        sage: dcrossing([Set((1,2))])
        -1
        sage: dcrossing(Set([Set((1,2))]))
        -1

    Set partition with block of size >= 3 is always at least
    0-dcrossing::

        sage: dcrossing(setp_to_edges(Set([Set((1,2,3))])))
        0
    """
    d = -1
    m = list(m_)
    while m:
        e1_ = m.pop()
        for e2_ in m:
            e1, e2 = sorted(e1_), sorted(e2_)
            if (e1[0] < e2[0] and e2[0] <= e1[1] and e1[1] < e2[1] and
                    e1[1] - e2[0] > d):
                d = e1[1] - e2[0]
            if (e2[0] < e1[0] and e1[0] <= e2[1] and e2[1] < e1[1] and
                    e2[1] - e1[0] > d):
                d = e2[1] - e1[0]
    return d


def setp_to_edges(p):
    """
    Transform a set partition into a list of edges.

    INPUT:

    - ``p`` -- a Sage set partition

    OUTPUT:

    A list of non-loop edges of the set partition. As this code just
    works with crossings, we can ignore the loops.

    EXAMPLES:

    The main example from the paper::

        sage: from sage.tests.arxiv_0812_2725 import *
        sage: sorted(setp_to_edges(Set(map(Set, [[1,5],[2,4,9],[3],[6,12],[7,10,11],[8]]))))
        [[1, 5], [2, 4], [4, 9], [6, 12], [7, 10], [10, 11]]
    """
    q = (sorted(b) for b in p)
    return [b[n: n + 2] for b in q for n in range(len(b) - 1)]


def dcrossvec_setp(n):
    """
    Return a list with the distribution of k-dcrossings on set partitions of [1..n].

    INPUT:

    - ``n`` -- nonnegative integer

    OUTPUT:

    A list whose k'th entry is the number of set partitions p for
    which dcrossing(p) = k. For example, let L = dcrossvec_setp(3).
    We have L = [1, 0, 4]. L[0] is 1 because there's 1 partition of
    [1..3] that has 0-dcrossing: [(1, 2, 3)].

    One tricky bit is that noncrossing matchings get put at the end,
    because L[-1] is the last element of the list. Above, we have
    L[-1] = 4 because the other four set partitions are all
    d-noncrossing. Because of this, you should not think of the last
    element of the list as having index n-1, but rather -1.

    EXAMPLES::

        sage: from sage.tests.arxiv_0812_2725 import *
        sage: dcrossvec_setp(3)
        [1, 0, 4]

        sage: dcrossvec_setp(4)
        [5, 1, 0, 9]

    The one set partition of 1 element is noncrossing, so the last
    element of the list is 1::

        sage: dcrossvec_setp(1)
        [1]
    """
    vec = [0] * n
    for p in SetPartitions(n):
        vec[dcrossing(setp_to_edges(p))] += 1
    return vec


def dcrossvec_cm(n):
    """
    Return a list with the distribution of k-dcrossings on complete matchings on n vertices.

    INPUT:

    - ``n`` -- nonnegative integer

    OUTPUT:

    A list whose k'th entry is the number of complete matchings m
    for which dcrossing(m) = k. For example, let L =
    dcrossvec_cm(4). We have L = [0, 1, 0, 2]. L[1] is 1 because
    there's one matching on 4 vertices that is 1-dcrossing: [(2, 4),
    (1, 3)]. L[0] is zero because dcrossing() returns the *largest*
    k for which the matching has a dcrossing, and 0-dcrossing is
    equivalent to 1-dcrossing for complete matchings.

    One tricky bit is that noncrossing matchings get put at the end,
    because L[-1] is the last element of the list. Because of this, you
    should not think of the last element of the list as having index
    n-1, but rather -1.

    If n is negative, you get silly results. Don't use them in your
    next paper. :)

    EXAMPLES:

    The single complete matching on 2 vertices has no crossings, so the
    only nonzero entry of the list (the last entry) is 1::

        sage: from sage.tests.arxiv_0812_2725 import *
        sage: dcrossvec_cm(2)
        [0, 1]

    Similarly, the empty matching has no crossings::

        sage: dcrossvec_cm(0)
        [1]

    For odd n, there are no complete matchings, so the list has all
    zeros::

        sage: dcrossvec_cm(5)
        [0, 0, 0, 0, 0]

        sage: dcrossvec_cm(4)
        [0, 1, 0, 2]
    """
    vec = [0] * max(n, 1)
    for m in CompleteMatchings(n):
        vec[dcrossing(m)] += 1
    return vec


def tablecolumn(n, k):
    """
    Return column n of Table 1 or 2 from the paper :arxiv:`0812.2725`.

    INPUT:

    - ``n`` -- positive integer

    - ``k`` -- integer for which table you want: Table 1 is complete
      matchings, Table 2 is set partitions

    OUTPUT:

    The n'th column of the table as a list. This is basically just the
    partial sums of dcrossvec_{cm,setp}(n).

    table2column(1, 2) incorrectly returns [], instead of [1], but you
    probably don't need this function to work through n = 1.

    EXAMPLES:

    Complete matchings::

        sage: from sage.tests.arxiv_0812_2725 import *
        sage: tablecolumn(2, 1)
        [1]

        sage: tablecolumn(6, 1)
        [5, 5, 11, 14, 15]

    Set partitions::

        sage: tablecolumn(5, 2)
        [21, 42, 51, 52]

        sage: tablecolumn(2, 2)
        [2]
    """
    if k == 1:
        v = dcrossvec_cm(n)
    else:
        v = dcrossvec_setp(n)
    i = v[-1]
    return [i + sum(v[:k]) for k in range(len(v) - 1)]



"""
This file (./mpoly_doctest.sage) was *autogenerated* from ./mpoly.tex,
with sagetex.sty version 2011/05/27 v2.3.1.
It contains the contents of all the sageexample environments from this file.
You should be able to doctest this file with:
sage -t ./mpoly_doctest.sage
It is always safe to delete this file; it is not used in typesetting your
document.

Sage example in ./mpoly.tex, line 65::

  sage: R = PolynomialRing(QQ, 'x,y,z')
  sage: x,y,z = R.gens() # gives the tuples of indeterminates

Sage example in ./mpoly.tex, line 78::

  sage: R = PolynomialRing(QQ, 'x', 10)

Sage example in ./mpoly.tex, line 84::

  sage: x = R.gens()
  sage: sum(x[i] for i in range(5))
  x0 + x1 + x2 + x3 + x4

Sage example in ./mpoly.tex, line 96::

  sage: def test_poly(ring, deg=3):
  ....:     monomials = Subsets(
  ....:         flatten([(x,)*deg for x in (1,) + ring.gens()]),
  ....:         deg, submultiset=True)
  ....:     return add(mul(m) for m in monomials)

Sage example in ./mpoly.tex, line 103::

  sage: test_poly(QQ['x,y'])
  x^3 + x^2*y + x*y^2 + y^3 + x^2 + x*y + y^2 + x + y + 1
  sage: test_poly(QQ['y,x'])
  y^3 + y^2*x + y*x^2 + x^3 + y^2 + y*x + x^2 + y + x + 1
  sage: test_poly(QQ['x,y']) == test_poly(QQ['y,x'])
  True

Sage example in ./mpoly.tex, line 127::

  sage: test_poly(PolynomialRing(QQ, 'x,y', order='deglex'))
  x^3 + x^2*y + x*y^2 + y^3 + x^2 + x*y + y^2 + x + y + 1

Sage example in ./mpoly.tex, line 230::

  sage: R.<x,y> = InfinitePolynomialRing(ZZ, order='lex')
  sage: p = mul(x[k] - y[k] for k in range(2)); p
  x_1*x_0 - x_1*y_0 - x_0*y_1 + y_1*y_0
  sage: p + x[100]
  x_100 + x_1*x_0 - x_1*y_0 - x_0*y_1 + y_1*y_0

Sage example in ./mpoly.tex, line 343::

  sage: R.<x,y,z> = QQ[]
  sage: p = 7*y^2*x^2 + 3*y*x^2 + 2*y*z + x^3 + 6
  sage: p.lt()
  7*x^2*y^2

Sage example in ./mpoly.tex, line 358::

  sage: p[x^2*y] == p[(2,1,0)] == p[2,1,0] == 3
  True

Sage example in ./mpoly.tex, line 366::

  sage: p(0, 3, -1)
  0
  sage: p.subs(x = 1, z = x^2+1)
  2*x^2*y + 7*y^2 + 5*y + 7

Sage example in ./mpoly.tex, line 381::

  sage: print("total={d}    (in x)={dx}    partial={ds}"\
  ....:   .format(d=p.degree(), dx=p.degree(x), ds=p.degrees()))
  total=4    (in x)=3    partial=(3, 2, 1)

Sage example in ./mpoly.tex, line 441::

  sage: R.<x,y> = QQ[]; p = x^2 + y^2; q = x + y
  sage: print("({quo})*({q}) + ({rem}) == {p}".format( \
  ....:         quo=p//q, q=q, rem=p%q, p=p//q*q+p%q))
  (-x + y)*(x + y) + (2*x^2) == x^2 + y^2
  sage: p.mod(q)  # is NOT equivalent to p%q
  2*y^2

Sage example in ./mpoly.tex, line 459::

  sage: R.<x,y> = QQ[exp(2*I*pi/5)][]
  sage: (x^10 + y^5).gcd(x^4 - y^2)
  x^2 + y
  sage: (x^10 + y^5).factor()
  (x^2 + y) * (x^2 + (a^3)*y) * (x^2 + (a^2)*y) * (x^2 + a*y) * (x^2 + (-a^3 - a^2 - a - 1)*y)

Sage example in ./mpoly.tex, line 564::

  sage: R.<x,y,z> = QQ[]
  sage: J = R.ideal(x^2 * y * z - 18,
  ....:             x * y^3 * z - 24,
  ....:             x * y * z^4 - 6)

Sage example in ./mpoly.tex, line 575::

  sage: J.dimension()
  0

Sage example in ./mpoly.tex, line 584::

  sage: J.variety()
  [{z: 1, y: 2, x: 3}]

Sage example in ./mpoly.tex, line 596::

  sage: V = J.variety(QQbar)
  sage: len(V)
  17

Sage example in ./mpoly.tex, line 603::

  sage: sorted(V, key=str)[-3:]
  [{z: 0.9324722294043558? + 0.3612416661871530?*I,
   y: -1.700434271459229? - 1.052864325754712?*I,
   x: 1.337215067329615? + 2.685489874065187?*I},
  {z: 0.9324722294043558? - 0.3612416661871530?*I,
   y: -1.700434271459229? + 1.052864325754712?*I,
   x: 1.337215067329615? - 2.685489874065187?*I},
   {z: 1, y: 2, x: 3}]

Sage example in ./mpoly.tex, line 619::

  sage: (xx, yy, zz) = QQbar['x,y,z'].gens()
  sage: sorted([pt[xx].degree() for pt in V])
  [1, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16,
  16, 16]

Sage example in ./mpoly.tex, line 640::

  sage: Set(tuple(abs(pt[i]) for i in (xx,yy,zz)) for pt in V)
  {(3, 2, 1)}

Sage example in ./mpoly.tex, line 662::

  sage: w = QQbar.zeta(17); w  # primitive root of 1
  0.9324722294043558? + 0.3612416661871530?*I
  sage: Set(pt[zz] for pt in V) == Set(w^i for i in range(17))
  True

Sage example in ./mpoly.tex, line 690::

  sage: set(pt[zz].minpoly() for pt in sorted(V, key=str)[:-1])
    {x^16 + x^15 + x^14 + x^13 + x^12 + x^11 + x^10 + x^9 + x^8 + x^7 + x^6 + x^5 + x^4 + x^3 + x^2 + x + 1}

Sage example in ./mpoly.tex, line 706::

  sage: def polar_form(z):
  ....:     rho = z.abs(); rho.simplify()
  ....:     theta = 2 * pi * z.rational_argument()
  ....:     return (SR(rho) * exp(I*theta))
  sage: sorted([tuple(polar_form(pt[i]) for i in [xx,yy,zz])
  ....:  for pt in sorted(V, key=str)[-3:]])
  [(3*e^(6/17*I*pi), 2*e^(-14/17*I*pi), e^(2/17*I*pi)),
   (3*e^(-6/17*I*pi), 2*e^(14/17*I*pi), e^(-2/17*I*pi)),
   (3, 2, 1)]

Sage example in ./mpoly.tex, line 733::

  sage: J.triangular_decomposition()
  [Ideal (z^17 - 1, y - 2*z^10, x - 3*z^3) of Multivariate
  Polynomial Ring in x, y, z over Rational Field]
  sage: J.transformed_basis()
  [z^17 - 1, -2*z^10 + y, -3/4*y^2 + x]

Sage example in ./mpoly.tex, line 909::

  sage: R.<x,y> = QQ[]
  sage: J = R.ideal(x^2 + y^2 - 1, 16*x^2*y^2 - 1)

Sage example in ./mpoly.tex, line 917::

  sage: ybar2 = R.quo(J)(y^2)
  sage: [ybar2^i for i in range(3)]
  [1, ybar^2, ybar^2 - 1/16]
  sage: ((ybar2 + 1)^2).lift()
  3*y^2 + 15/16

Sage example in ./mpoly.tex, line 958::

  sage: u = (16*y^4 - 16*y^2 + 1).lift(J); u
  [16*y^2, -1]
  sage: u[0]*J.0 + u[1]*J.1
  16*y^4 - 16*y^2 + 1

Sage example in ./mpoly.tex, line 968::

  sage: (y^4).mod(J)
  y^2 - 1/16

Sage example in ./mpoly.tex, line 978::

  sage: (y^4).reduce([x^2 + y^2 - 1, 16*x^2*y^2 - 1])
  y^4

Sage example in ./mpoly.tex, line 1047::

  sage: 1 in ideal(x^2+y^2-1, (x-4)^2+y^2-1)
  False

Sage example in ./mpoly.tex, line 1056::

  sage: R(1).lift(ideal(x^2+y^2-1, (x-4)^2+y^2-1, x-y))
  [-1/28*y + 1/14, 1/28*y + 1/14, -1/7*x + 1/7*y + 4/7]

Sage example in ./mpoly.tex, line 1079::

  sage: J1 = (x^2 + y^2 - 1, 16*x^2*y^2 - 1)*R
  sage: J2 = (x^2 + y^2 - 1,  4*x^2*y^2 - 1)*R
  sage: J1.radical() == J1
  True
  sage: J2.radical()
  Ideal (2*y^2 - 1, 2*x^2 - 1) of Multivariate Polynomial
  Ring in x, y over Rational Field
  sage: 2*y^2 - 1 in J2
  False

Sage example in ./mpoly.tex, line 1122::

  sage: C = ideal(x^2 + y^2 - 1); H = ideal(16*x^2*y^2 - 1)
  sage: C + H == J1
  True

Sage example in ./mpoly.tex, line 1151::

  sage: CH = C.intersection(H).quotient(ideal(4*x*y-1)); CH
  Ideal (4*x^3*y + 4*x*y^3 + x^2 - 4*x*y + y^2 - 1) of
  Multivariate Polynomial Ring in x, y over Rational Field
  sage: CH.gen(0).factor()
  (4*x*y + 1) * (x^2 + y^2 - 1)

Sage example in ./mpoly.tex, line 1161::

  sage: H.quotient(C) == H
  True

Sage example in ./mpoly.tex, line 1184::

  sage: [J.dimension() for J in [J1, J2, C, H, H*J2, J1+J2]]
  [0, 0, 1, 1, 1, -1]

Sage example in ./mpoly.tex, line 1285::

  sage: R.<x,y,z> = QQ[]
  sage: J = ideal(2*x+y-2*z, 2*x+2*y+z-1)
  sage: J.elimination_ideal(x)
  Ideal (y + 3*z - 1) of Multivariate Polynomial Ring in x, y, z
  over Rational Field
  sage: J.elimination_ideal([x,y])
  Ideal (0) of Multivariate Polynomial Ring in x, y, z over Rational Field

Sage example in ./mpoly.tex, line 1312::

  sage: R.<x,y> = QQ[]
  sage: J1 = ideal(x^2 + y^2 - 1, 16*x^2*y^2 - 1)

Sage example in ./mpoly.tex, line 1328::

  sage: g = J1.elimination_ideal(y).gens(); g
  [16*x^4 - 16*x^2 + 1]
  sage: SR(g[0]).solve(SR(x)) # solves by radicals
  [x == -1/2*sqrt(sqrt(3) + 2), x == 1/2*sqrt(sqrt(3) + 2),
  x == -1/2*sqrt(-sqrt(3) + 2), x == 1/2*sqrt(-sqrt(3) + 2)]

Sage example in ./mpoly.tex, line 1351::

  sage: C.elimination_ideal(y).gens()
  [0]
  sage: H.elimination_ideal(y).gens()
  [0]

Sage example in ./mpoly.tex, line 1431::

  sage: R.<x,y,t> = QQ[]
  sage: Param = R.ideal((1-t^2)-(1+t^2)*x, 2*t-(1+t^2)*y)

Sage example in ./mpoly.tex, line 1437::

  sage: Param.elimination_ideal(t).gens()
  [x^2 + y^2 - 1]

Sage example in ./mpoly.tex, line 1469::

  sage: R.<x,y,t> = QQ[]
  sage: eq = x^2 + (y-t)^2 - 1/2*(t^2+1)
  sage: fig = add((eq(t=k/5)*QQ[x,y]).plot() for k in (-15..15))
  sage: fig.show(aspect_ratio=1, xmin=-2, xmax=2, ymin=-3, ymax=3)

Sage example in ./mpoly.tex, line 1494::

  sage: env = ideal(eq, eq.derivative(t)).elimination_ideal(t)
  sage: env.gens()
  [2*x^2 - 2*y^2 - 1]

Sage example in ./mpoly.tex, line 1502::

  sage: env.change_ring(QQ[x,y]).plot((x,-2,2),(y,-3,3))
  Graphics object consisting of 1 graphics primitive

Sage example in ./mpoly.tex, line 1538::

  sage: R.<x,y,t> = QQ[]
  sage: J = (y-t*x, y-t*(1-x))*R
  sage: (x^2+y^2) - ((1-x)^2+y^2) in J
  False

Sage example in ./mpoly.tex, line 1553::

  sage: R.<x,y,t,u> = QQ[]
  sage: J = (y-t*x, y-t*(1-x), t*u-1)*R
  sage: (x^2+y^2) - ((1-x)^2+y^2) in J
  True

Sage example in ./mpoly.tex, line 1615::

  sage: R.<x,y,t> = QQ[]

Sage example in ./mpoly.tex, line 1618::

  sage: eq.derivative(t).resultant(eq, t)
  x^2 - y^2 - 1/2

Sage example in ./mpoly.tex, line 1636::

  sage: R.<x,y> = QQ[]
  sage: p = y^2 - x; q = y^2 + x
  sage: p.resultant(q, y)
  4*x^2
  sage: ideal(p, q).elimination_ideal(y)
  Ideal (x) of Multivariate Polynomial Ring in x, y over Rational Field

Sage example in ./mpoly.tex, line 1665::

  sage: R.<x,y> = QQ[]
  sage: ((x^2 + y^2)*(x^2 + y^2 + 1)*R).dimension()
  1

Sage example in ./mpoly.tex, line 1690::

  sage: R.<x,y> = QQ[]
  sage: J1 = (x^2 + y^2 - 1, 16*x^2*y^2 - 1)*R
  sage: J1.variety()
  []

Sage example in ./mpoly.tex, line 1705::

  sage: J1.variety(QQbar)[0:2]
  [{y: -0.9659258262890683?, x: -0.2588190451025208?},
  {y: -0.9659258262890683?, x: 0.2588190451025208?}]

Sage example in ./mpoly.tex, line 1759::

  sage: R.<x,y> = PolynomialRing(QQ, order='lex')
  sage: C = ideal(x^2+y^2-1)
  sage: D = ideal((x+y-1)*(x+y+1))
  sage: J = C + D

Sage example in ./mpoly.tex, line 1786::

  sage: J.triangular_decomposition()
  [Ideal (y, x^2 - 1) of Multivariate Polynomial Ring in x, y
  over Rational Field,
  Ideal (y^2 - 1, x) of Multivariate Polynomial Ring in x, y
  over Rational Field]

Sage example in ./mpoly.tex, line 1840::

  sage: D = ideal((x+2*y-1)*(x+2*y+1)); J = C + D
  sage: J.variety()
  [{y: 0, x: 1}, {y: 0, x: -1}, {y: 4/5, x: -3/5}, {y: -4/5, x: 3/5}]
  sage: [T.gens() for T in J.triangular_decomposition()]
  [[y, x^2 - 1], [25*y^2 - 16, 4*x + 3*y]]

Sage example in ./mpoly.tex, line 1855::

  sage: Jy = J.elimination_ideal(x); Jy.gens()
  [25*y^3 - 16*y]

Sage example in ./mpoly.tex, line 1863::

  sage: ys = QQ['y'](Jy.0).roots(); ys
  [(4/5, 1), (0, 1), (-4/5, 1)]
  sage: QQ['x'](J.1(y=ys[0][0])).roots()
  [(-3/5, 1), (-13/5, 1)]

Sage example in ./mpoly.tex, line 1882::

  sage: ys = CDF['y'](Jy.0).roots(); ys # abs tol 2e-15
  [(-0.8, 1), (0.0, 1), (0.8, 1)]
  sage: [CDF['x'](p(y=ys[0][0])).roots() for p in J.gens()] # abs tol 2e-15
  [[(-0.5999999999999999, 1), (0.6000000000000001, 1)], [(0.6000000000000001, 1), (2.600000000000001, 1)]]

Sage example in ./mpoly.tex, line 1911::

  sage: R.<x,y> = QQ[]; J = ideal([ x^7-(100*x-1)^2, y-x^7+1 ])
  sage: J.variety(RealField(51))  # random
  [{y: 396340.890166545, x: -14.1660266425312}]

Sage example in ./mpoly.tex, line 1923::

  sage: J.variety(AA)
  [{y: -0.999999999999990?, x: 0.00999999900000035?},
  {y: -0.999999999999990?, x: 0.01000000100000035?},
  {y: 396340.8901665450?, x: 6.305568998641385?}]

Sage example in ./mpoly.tex, line 1983::

  sage: len(J2.variety(QQbar)), J2.vector_space_dimension()
  (4, 8)

Sage example in ./mpoly.tex, line 1993::

  sage: J2.normal_basis()
  [x*y^3, y^3, x*y^2, y^2, x*y, y, x, 1]

Sage example in ./mpoly.tex, line 2187::

  sage: R.<x,y,z,t> = PolynomialRing(QQ, order='lex')

Sage example in ./mpoly.tex, line 2243::

  sage: ((x+y+z)^2).reduce([x-t, y-t^2, z^2-t])
  2*z*t^2 + 2*z*t + t^4 + 2*t^3 + t^2 + t

Sage example in ./mpoly.tex, line 2298::

  sage: R.<x,y> = PolynomialRing(QQ, order='lex')
  sage: (g, h) = (x-y, x-y^2);  p = x*y - x
  sage: p.reduce([g, h])  # two reductions by h
  y^3 - y^2
  sage: p.reduce([h, g])  # two reductions by g
  y^2 - y

Sage example in ./mpoly.tex, line 2311::

  sage: p - y*g + h
  0

Sage example in ./mpoly.tex, line 2575::

  sage: R.<x,y> = PolynomialRing(QQ, order='lex')
  sage: R.ideal(x*y^4, x^2*y^3, x^4*y, x^5).basis_is_groebner()
  True

Sage example in ./mpoly.tex, line 2584::

  sage: R.ideal(x^2+y^2-1, 16*x^2*y^2-1).basis_is_groebner()
  False

Sage example in ./mpoly.tex, line 2610::

  sage: R.ideal(x^2+y^2-1, 16*x^2*y^2-1).groebner_basis()
  [x^2 + y^2 - 1, y^4 - y^2 + 1/16]

Sage example in ./mpoly.tex, line 2618::

  sage: R.ideal(16*x^2*y^2-1).groebner_basis()
  [x^2*y^2 - 1/16]

Sage example in ./mpoly.tex, line 2626::

  sage: R.ideal(x^2+y^2-1, (x+y)^2-1).groebner_basis()
  [x^2 + y^2 - 1, x*y, y^3 - y]

Sage example in ./mpoly.tex, line 2636::

  sage: R_lex.<x,y> = PolynomialRing(QQ, order='lex')
  sage: J_lex = (x*y+x+y^2+1, x^2*y+x*y^2+1)*R_lex; J_lex.gens()
  [x*y + x + y^2 + 1, x^2*y + x*y^2 + 1]
  sage: J_lex.groebner_basis()
  [x - 1/2*y^3 + y^2 + 3/2, y^4 - y^3 - 3*y - 1]

Sage example in ./mpoly.tex, line 2644::

  sage: R_invlex = PolynomialRing(QQ, 'x,y', order='invlex')
  sage: J_invlex = J_lex.change_ring(R_invlex); J_invlex.gens()
  [y^2 + x*y + x + 1, x*y^2 + x^2*y + 1]
  sage: J_invlex.groebner_basis()
  [y^2 + x*y + x + 1, x^2 + x - 1]

Sage example in ./mpoly.tex, line 2651::

  sage: R_drl = PolynomialRing(QQ, 'x,y', order='degrevlex')
  sage: J_drl = J_lex.change_ring(R_drl); J_drl.gens()
  [x*y + y^2 + x + 1, x^2*y + x*y^2 + 1]
  sage: J_drl.groebner_basis()
  [y^3 - 2*y^2 - 2*x - 3, x^2 + x - 1, x*y + y^2 + x + 1]

Sage example in ./mpoly.tex, line 2719::

  sage: p = (x + y)^5
  sage: J_lex.reduce(p)
  17/2*y^3 - 12*y^2 + 4*y - 49/2

Sage example in ./mpoly.tex, line 2726::

  sage: p.reduce(J_lex.groebner_basis())
  17/2*y^3 - 12*y^2 + 4*y - 49/2

Sage example in ./mpoly.tex, line 2732::

  sage: R_lex.quo(J_lex)(p)
  17/2*ybar^3 - 12*ybar^2 + 4*ybar - 49/2

Sage example in ./mpoly.tex, line 2738::

  sage: R_drl.quo(J_drl)(p)
  5*ybar^2 + 17*xbar + 4*ybar + 1

Sage example in ./mpoly.tex, line 2751::

  sage: J_lex.normal_basis()
  [y^3, y^2, y, 1]
  sage: J_invlex.normal_basis()
  [x*y, y, x, 1]
  sage: J_drl.normal_basis()
  [y^2, y, x, 1]

Sage example in ./mpoly.tex, line 2775::

  sage: ideal(16*x^2*y^2-1).dimension()
  1

Sage example in ./mpoly.tex, line 2851::

  sage: R.<t,x,y,z> = PolynomialRing(QQ, order='lex')
  sage: J = R.ideal(t+x+y+z-1, t^2-x^2-y^2-z^2-1, t-x*y)
  sage: [u.polynomial(u.variable(0)) for u in J.groebner_basis()]
  [t + x + y + z - 1,
  (y + 1)*x + y + z - 1,
  (z - 2)*x + y*z - 2*y - 2*z + 1,
  (z - 2)*y^2 + (-2*z + 1)*y - z^2 + z - 1]

Sage example in ./mpoly.tex, line 2970::

  sage: from sage.rings.ideal import Cyclic
  sage: Cyclic(QQ['x,y,z'])
  Ideal (x + y + z, x*y + x*z + y*z, x*y*z - 1) of
  Multivariate Polynomial Ring in x, y, z over Rational Field

Sage example in ./mpoly.tex, line 2980::

  sage: def C(R, n): return Cyclic(PolynomialRing(R, 'x', n))

Sage example in ./mpoly.tex, line 3010::

  sage: p = previous_prime(2^30)
  sage: len(C(GF(p), 6).groebner_basis())
  45
"""



r"""
Simple profiling tool

AUTHORS:

- David Harvey (August 2006)
- Martin Albrecht
"""

# *****************************************************************************
#       Copyright (C) 2006 William Stein <wstein@gmail.com>
#                     2006 David Harvey <dmharvey@math.harvard.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
# *****************************************************************************

from sage.misc.timing import cputime
import inspect
import sys


class Profiler:
    r"""
    Keeps track of CPU time used between a series of user-defined checkpoints.

    It's probably not a good idea to use this class in an inner loop :-)

    EXAMPLES::

        from sage.misc.profiler import Profiler
        sage: def f():                        # not tested
        ....:     p = Profiler()

    Calling ``p(message)`` creates a checkpoint::

        sage: p("try factoring 15")           # not tested

    Do something time-consuming::

        sage: x = factor(15)                  # not tested

    You can create a checkpoints without a string; ``Profiler``
    will use the source code instead::

        sage: # not tested
        sage: p()
        sage: y = factor(25)
        sage: p("last step")
        sage: z = factor(35)
        sage: p()

    This will give a nice list of timings between checkpoints::

        sage: print(p)                        # not tested

    Let's try it out::

        sage: f()                             # not tested
            3.020s -- try factoring 15
           15.240s -- line 17: y = factor(25)
         5000.190s -- last step

    .. SEEALSO:: :func:`runsnake`

    .. TODO::

        - Add Pyrex source code inspection (I assume it doesn't
          currently do this)
        - Add ability to sort output by time
        - Add option to constructor to print timing immediately when
          checkpoint is reached
        - Migrate to Pyrex?
        - Add ability to return timings in a more machine-friendly
          format

    AUTHOR:

    - David Harvey (August 2006)
    """

    def __init__(self, systems=[], verbose=False):
        """
        INPUT:

        - ``systems`` -- list of interfaces to other system which implements a
          cputime method. The cputimes of all provided systems will be added
          to the cputime of Sage itself.
        """
        systems = [e.cputime for e in systems]
        self._cputime_functions = [cputime] + list(systems)
        self._verbose = bool(verbose)
        self.clear()

    def clear(self):
        # _checkpoints is list of pairs (details, time), where time is a float
        # and details is a triple (line_number, context, message)
        self._checkpoints = []
        self._active_details = None   # details from the last __call__() call
        self._last_cputime = [None]*len(self._cputime_functions)

    def __call__(self, message=None):
        """ Adds a checkpoint. """
        entry_times = [fn() for fn in self._cputime_functions]

        frame = inspect.currentframe().f_back
        try:
            frame_info = inspect.getframeinfo(frame, 9)
            line_number = frame_info[1]
            context = frame_info[3]
        finally:
            # This is here to prevent reference cycles
            # (according to the warning in the python documentation:
            # http://docs.python.org/lib/inspect-stack.html):
            del frame_info
            del frame

        if self._active_details is not None:
            _time = sum([entry_times[i]-self._last_cputime[i] for i in range(len(entry_times))])
            self._checkpoints.append((self._active_details, _time))

        self._active_details = (line_number, context, message)

        self._last_cputime = [fn() for fn in self._cputime_functions]
        if self._verbose:
            print(self.print_last())
            sys.stdout.flush()

    def __repr__(self):
        """ Returns a nicely formatted table of stored checkpoints and timings. """
        if not self._checkpoints:
            return "no checkpoints defined"

        output = []
        for ((line_number, context, message), time_used) in self._checkpoints:
            if message is None:
                # If the user hasn't given a message, we look for some
                # source code to print instead
                found = "(unknown)"
                for line in context[5:]:
                    line = line.strip()
                    if line != "":
                        found = line
                        break

                if len(found) > 60:
                    found = found[:60] + "..."   # in case the source line is really long
                message = "line %d: %s" % (line_number, found)

            output.append("%9.3fs -- %s" % (time_used, message))

        return "\n".join(output)

    def print_last(self):
        """
        Print the last profiler step.
        """
        if not self._checkpoints:
            return ""

        ((line_number, context, message), time_used) = self._checkpoints[-1]
        if message is None:
            # If the user hasn't given a message, we look for some
            # source code to print instead
            found = "(unknown)"
            for line in context[5:]:
                line = line.strip()
                if line != "":
                    found = line
                    break

            if len(found) > 60:
                found = found[:60] + "..."   # in case the source line is really long
            message = "line %d: %s" % (line_number, found)

        return "%9.3fs -- %s" % (time_used, message)

# end of file



"""
This file (./sol/recequadiff_doctest.sage) was *autogenerated* from ./sol/recequadiff.tex,
with sagetex.sty version 2011/05/27 v2.3.1.
It contains the contents of all the sageexample environments from this file.
You should be able to doctest this file with:
sage -t ./sol/recequadiff_doctest.sage
It is always safe to delete this file; it is not used in typesetting your
document.

Sage example in ./sol/recequadiff.tex, line 16::

  sage: x = var('x')
  sage: y = function('y')(x)
  sage: ed = (desolve(y*diff(y,x)/sqrt(1+y^2) == sin(x),y)); ed
  sqrt(y(x)^2 + 1) == _C - cos(x)

Sage example in ./sol/recequadiff.tex, line 27::

  sage: c = ed.variables()[0]
  sage: assume(c-cos(x) > 0)
  sage: sol = solve(ed,y) ; sol
  [y(x) == -sqrt(_C^2 - 2*_C*cos(x) + cos(x)^2 - 1),
   y(x) == sqrt(_C^2 - 2*_C*cos(x) + cos(x)^2 - 1)]

Sage example in ./sol/recequadiff.tex, line 34::

  sage: P = Graphics()
  sage: for j in [0,1]:
  ....:   for k in range(0,20,2):
  ....:     P += plot(sol[j].substitute(c==2+0.25*k).rhs(),x,-3,3)
  sage: P
  Graphics object consisting of 20 graphics primitives

Sage example in ./sol/recequadiff.tex, line 52::

  sage: sol = desolve(diff(y,x)==sin(x)/cos(y), y, show_method=True)
  sage: sol
  [sin(y(x)) == _C - cos(x), 'separable']
  sage: solve(sol[0],y)
  [y(x) == -arcsin(-_C + cos(x))]

Sage example in ./sol/recequadiff.tex, line 80::

  sage: x = var('x')
  sage: y = function('y')(x)
  sage: id(x) = x
  sage: u = function('u')(x)
  sage: d = diff(u*id,x)
  sage: DE = (x*y*d == x**2+y**2).substitute(y == u*id)
  sage: eq = desolve(DE,u)
  sage: sol = solve(eq,u)
  sage: sol
  [u(x) == -sqrt(2*_C + 2*log(x)), u(x) == sqrt(2*_C + 2*log(x))]
  sage: Y = [x*sol[0].rhs() , x*sol[1].rhs()]
  sage: Y[0]
  -sqrt(2*_C + 2*log(x))*x
"""



from sage.tests.memcheck.verify_no_leak import verify_no_leak


def test_sqrt_sqrt_2() -> None:
    from sage.misc.functional import sqrt
    T2 = sqrt(2)

    def sqrt_T2() -> None:
        sqrt(T2)

    verify_no_leak(sqrt_T2)



"""
This file (./sol/domaines_doctest.sage) was *autogenerated* from ./sol/domaines.tex,
with sagetex.sty version 2011/05/27 v2.3.1.
It contains the contents of all the sageexample environments from this file.
You should be able to doctest this file with:
sage -t ./sol/domaines_doctest.sage
It is always safe to delete this file; it is not used in typesetting your
document.

Sage example in ./sol/domaines.tex, line 4::

  sage: def ndigits(x): return x.ndigits()
  sage: o = 720; ndigits(o)
  3

Sage example in ./sol/domaines.tex, line 28::

  sage: a = Reals(17)(pi); b = Reals(42)(pi)
  sage: type(a) == type(b)
  True
  sage: parent(a), parent(b)
  (Real Field with 17 bits of precision, Real Field with 42 bits of precision)

Sage example in ./sol/domaines.tex, line 46::

  sage: a = 0.1; b = 0.1*1
  sage: type(a), type(b)
  (<... 'sage.rings.real_mpfr.RealLiteral'>, <... 'sage.rings.real_mpfr.RealNumber'>)
  sage: parent(a) == parent(b)
  True

Sage example in ./sol/domaines.tex, line 70::

  sage: Reals(100)(a)-1/10
  0.00000000000000000000000000000
  sage: Reals(100)(b)-1/10
  5.5511151231257629805955278152e-18

Sage example in ./sol/domaines.tex, line 80::

  sage: E = CombinatorialFreeModule(QQ, [1,2,3])
  sage: H = Hom(E,E); H.rename('H')
  sage: C = E.category(); C
  Category of finite dimensional vector spaces with basis over Rational Field
  sage: phi1 = E.module_morphism(on_basis=lambda i: E.term(i), codomain=E)
  sage: phi2 = E.module_morphism(on_basis=lambda i: E.term(i),
  ....:                          triangular='lower', codomain=E)
  sage: phi3 = E.module_morphism(diagonal=lambda i: 1, codomain=E,
  ....:                          category=C)
  sage: phi1.parent() == phi2.parent() == phi3.parent() == H
  True
  sage: type(phi1)
  <class 'sage.modules.with_basis.morphism.ModuleMorphismByLinearity_with_category'>
  sage: type(phi2)
  <class 'sage.modules.with_basis.morphism.TriangularModuleMorphismByLinearity_with_category'>
  sage: type(phi3)
  <class 'sage.modules.with_basis.morphism.DiagonalModuleMorphism_with_category'>
"""



"""
This file (./float_doctest.sage) was *autogenerated* from ./float.tex,
with sagetex.sty version 2011/05/27 v2.3.1.
It contains the contents of all the sageexample environments from this file.
You should be able to doctest this file with:
sage -t ./float_doctest.sage
It is always safe to delete this file; it is not used in typesetting your
document.

Sage example in ./float.tex, line 251::

  sage: xrdf = RDF(3.0)

Sage example in ./float.tex, line 276::

  sage: R100 = RealField(100) # precision: 100 bits.
  sage: x100 = R100(3/8); x100
  0.37500000000000000000000000000

Sage example in ./float.tex, line 300::

  sage: Rdefault = RealField()  # default precision of 53 bits
  sage: xdefault = Rdefault(2/3)

Sage example in ./float.tex, line 319::

  sage: xrdf.prec()
  53
  sage: x100.prec()
  100
  sage: xdefault.prec()
  53

Sage example in ./float.tex, line 347::

  sage: x = 1.0; type(x)
  <... 'sage.rings.real_mpfr.RealLiteral'>
  sage: x.prec()
  53

Sage example in ./float.tex, line 384::

  sage: x = 1.0         # x belongs to RealField()
  sage: x = 0.1e+1      # idem: x belongs to RealField()
  sage: x = 1           # x is an integer
  sage: x = RDF(1)      # x is a machine double-precision number
  sage: x = RDF(1.)     # idem: x is a machine double-precision number
  sage: x = RDF(0.1e+1) # idem
  sage: x = 4/3         # x is a rational number
  sage: R = RealField(20)
  sage: x = R(1)        # x is a 20-bit floating-point number

Sage example in ./float.tex, line 400::

  sage: RDF(8/3)
  2.6666666666666665
  sage: R100 = RealField(100); R100(8/3)
  2.6666666666666666666666666667

Sage example in ./float.tex, line 412::

  sage: x = R100(8/3)
  sage: R = RealField(); R(x)
  2.66666666666667
  sage: RDF(x)
  2.6666666666666665

Sage example in ./float.tex, line 431::

  sage: 1.0/0.0
  +infinity
  sage: RDF(1)/RDF(0)
  +infinity
  sage: RDF(-1.0)/RDF(0.)
  -infinity

Sage example in ./float.tex, line 441::

  sage: 0.0/0.0
  NaN
  sage: RDF(0.0)/RDF(0.0)
  NaN

Sage example in ./float.tex, line 496::

  sage: R2 = RealField(2)

Sage example in ./float.tex, line 545::

  sage: x2 = R2(1.); x2.ulp()
  0.50
  sage: xr = 1.; xr.ulp()
  2.22044604925031e-16

Sage example in ./float.tex, line 655::

  sage: a = 10000.0; b = 9999.5; c = 0.1; c
  0.100000000000000
  sage: a1 = a+c # add a small perturbation to a.
  sage: a1-b
  0.600000000000364

Sage example in ./float.tex, line 680::

  sage: a = 1.0; b = 10.0^4; c = 1.0
  sage: delta = b^2-4*a*c
  sage: x = (-b-sqrt(delta))/(2*a); y = (-b+sqrt(delta))/(2*a)
  sage: x, y
  (-9999.99990000000, -0.000100000001111766)

Sage example in ./float.tex, line 692::

  sage: x+y+b/a
  0.000000000000000
  sage: x*y-c/a
  1.11766307320238e-9

Sage example in ./float.tex, line 713::

  sage: y = (c/a)/x; y
  -0.000100000001000000
  sage: x+y+b/a
  0.000000000000000
  sage: x*y-c/a
  -1.11022302462516e-16

Sage example in ./float.tex, line 746::

  sage: x1 = R2(1/2); x2 = R2(4); x3 = R2(-4)
  sage: x1, x2, x3
  (0.50, 4.0, -4.0)
  sage: x1+(x2+x3)
  0.50
  sage: (x1+x2)+x3
  0.00

Sage example in ./float.tex, line 781::

  sage: x = RDF(1/3)
  sage: for i in range(1,100): x = 4*x-1; print(x)
  0.33333333333333326
  0.33333333333333304
  0.33333333333333215
  ...
  -1.0
  -5.0
  -21.0
  -85.0
  -341.0
  -1365.0
  -5461.0
  -21845.0
  ...

Sage example in ./float.tex, line 824::

  sage: x = RDF(1/2)
  sage: for i in range(1,100): x = 3*x-1; print(x)
  0.5
  0.5
  0.5
  ...
  0.5

Sage example in ./float.tex, line 863::

  sage: x = RDF(1/3)

Sage example in ./float.tex, line 867::

  sage: x = 1/3

Sage example in ./float.tex, line 990::

  sage: def sumharmo(p):
  ....:    RFP = RealField(p)
  ....:    y = RFP(1.); x = RFP(0.); n = 1
  ....:    while x != y:
  ....:        y = x; x += 1/n; n += 1
  ....:    return p, n, x

Sage example in ./float.tex, line 1003::

  sage: sumharmo(2)
  (2, 5, 2.0)
  sage: sumharmo(20)
  (20, 131073, 12.631)

Sage example in ./float.tex, line 1072::

  sage: def iter(y,delta,a,n):
  ....:     for i in range(0,n):
  ....:         y += delta
  ....:         delta *= a
  ....:     return y

Sage example in ./float.tex, line 1087::

  sage: def exact(y,delta,a,n):
  ....:     return y+delta*(1-a^n)/(1-a)

Sage example in ./float.tex, line 1106::

  sage: y0 = RDF(10^13); delta0 = RDF(1); a = RDF(1-10^(-8)); n = 100000
  sage: ii = iter(y0,delta0,a,n)
  sage: s = exact(10^13,1,1-10^(-8),n)
  sage: print("exact - classical summation: %.1f" % (s-ii))  # abs tol 0.1
  exact - classical summation: -45.6

Sage example in ./float.tex, line 1128::

  sage: def sumcomp(y,delta,e,n,a):
  ....:     for i in range(0,n):
  ....:         b = y
  ....:         e += delta
  ....:         y = b+e
  ....:         e += (b-y)
  ....:         delta = a*delta # new value of delta
  ....:     return y

Sage example in ./float.tex, line 1194::

  sage: c = sumcomp(y0,delta0,RDF(0.0),n,a)
  sage: print("exact - compensated summation: %.5f" \
              % RDF(s-RR(c).exact_rational()))
  exact - compensated summation: -0.00042

Sage example in ./float.tex, line 1242::

  sage: x = CDF(2,1.); x
  2.0 + 1.0*I
  sage: y = CDF(20,0); y
  20.0

Sage example in ./float.tex, line 1249::

  sage: z = ComplexDoubleElement(2.,1.); z
  2.0 + 1.0*I

Sage example in ./float.tex, line 1269::

  sage: C = ComplexField(); C(2,3)
  2.00000000000000 + 3.00000000000000*I
  sage: C100 = ComplexField(100); C100(2,3)
  2.0000000000000000000000000000 + 3.0000000000000000000000000000*I

Sage example in ./float.tex, line 1297::

  sage: R200 = RealField(200); R200.pi()
  3.1415926535897932384626433832795028841971693993751058209749
  sage: R200.euler_constant()
  0.57721566490153286060651209008240243104215933593992359880577

Sage example in ./float.tex, line 1314::

  sage: x = RDF.pi()/2; x.cos() # floating-point approximation of zero!
  6.123233995736757e-17
  sage: x.cos().arccos() - x
  0.0

Sage example in ./float.tex, line 1564::

  sage: r3 = RIF(sqrt(3)); r3
  1.732050807568877?
  sage: print(r3.str(style='brackets'))
  [1.7320508075688769 .. 1.7320508075688775]

Sage example in ./float.tex, line 1602::

  sage: sage.rings.real_mpfi.printing_style = 'brackets'

Sage example in ./float.tex, line 1616::

  sage: r2 = RIF(2); r2, r2.diameter()
  ([2.0000000000000000 .. 2.0000000000000000], 0.000000000000000)

Sage example in ./float.tex, line 1628::

  sage: rpi = RIF(sqrt(2),pi); rpi
  [1.4142135623730949 .. 3.1415926535897936]
  sage: RIF(0,+infinity)
  [0.0000000000000000 .. +infinity]

Sage example in ./float.tex, line 1649::

  sage: RBF(pi)
  [3.141592653589793 +/- ...e-16]
  sage: RealBallField(100)(pi)
  [3.14159265358979323846264338328 +/- ...e-30]

Sage example in ./float.tex, line 1662::

  sage: RBF(2).rad()
  0.00000000

Sage example in ./float.tex, line 1675::

  sage: si = sin(RIF(pi))
  sage: si.contains_zero()
  True
  sage: sb = sin(RBF(pi))
  sage: sb.contains_zero()
  True

Sage example in ./float.tex, line 1704::

  sage: a = RealIntervalField(30)(1, RR(1).nextabove())
  sage: a.bisection()
  ([1.0000000000 .. 1.0000000000], [1.0000000000 .. 1.0000000019])

Sage example in ./float.tex, line 1713::

  sage: b = RealIntervalField(2)(-1,6)
  sage: b.center(), b.diameter()
  (2.0, 8.0)

Sage example in ./float.tex, line 1725::

  sage: s = RIF(1,2)
  sage: b = RBF(s)
  sage: bpi = RBF(pi)
  sage: ipi = RIF(bpi)

Sage example in ./float.tex, line 1735::

  sage: RIF(RBF(RIF(1,2))) == RIF(1,2)
  False
  sage: RBF(RIF(RBF(pi))) == RBF(pi)
  False

Sage example in ./float.tex, line 1810::

  sage: E = RIF(-pi/4,pi)
  sage: sin(E)
  [-0.70710678118654769 .. 1.0000000000000000]
  sage: E = RIF(-1,2); exp(E)
  [0.36787944117144227 .. 7.3890560989306505]
  sage: E = RIF(0,1); log(E)
  [-infinity .. -0.0000000000000000]

Sage example in ./float.tex, line 1838::

  sage: E=RIF(-pi,pi)
  sage: f = lambda x: sin(x)/x
  sage: f(E)
  [-infinity .. +infinity]

Sage example in ./float.tex, line 1913::

  sage: x = RIF(-1,1)
  sage: 1-x^2
  [0.0000000000000000 .. 1.0000000000000000]
  sage: 1-x*x
  [0.0000000000000000 .. 2.0000000000000000]
  sage: (1-x)*(1+x)
  [0.0000000000000000 .. 4.0000000000000000]

Sage example in ./float.tex, line 1975::

  sage: def bisect(funct,x,tol,zeros):
  ....:     if 0 in funct(x):
  ....:         if x.diameter()>tol:
  ....:             x1,x2 = x.bisection()
  ....:             bisect(funct,x1,tol,zeros)
  ....:             bisect(funct,x2,tol,zeros)
  ....:         else:
  ....:             zeros.append(x)
  sage: sage.rings.real_mpfi.printing_style = 'question'
  sage: fs = lambda x: sin(1/x)
  sage: d = RealIntervalField(100)(1/64,1/32)
  sage: zeros = []
  sage: bisect(fs,d,10^(-25),zeros)
  sage: for s in zeros:
  ....:     s
  0.015915494309189533576888377?
  0.01675315190441003534409303?
  0.01768388256576614841876487?
  0.018724110951987686561045148?
  0.01989436788648691697111047?
  0.021220659078919378102517835?
  0.02273642044169933368126911?
  0.024485375860291590118289809?
  0.026525823848649222628147293?
  0.02893726238034460650343342?
  sage: dfs = lambda x: -cos(1/x)/x^2
  sage: not any(dfs(z).contains_zero() for z in zeros)
  True

Sage example in ./float.tex, line 2022::

  sage: def NearlySingularMatrix(R,n):
  ....:     M=matrix(R,n,n)
  ....:     for i in range(0,n):
  ....:         for j in range(0,n):
  ....:             M[i,j]= (1+log(R(1+i)))/((i+1)^2+(j+1)^2)
  ....:     return M

Sage example in ./float.tex, line 2044::

  sage: n=35
  sage: NearlySingularMatrix(RDF,n).det() == 0.0
  True

Sage example in ./float.tex, line 2053::

  sage: NearlySingularMatrix(RBF,n).det().contains_zero()
  True

Sage example in ./float.tex, line 2064::

  sage: def tryDet(R,n):
  ....:     p = 53
  ....:     z = True
  ....:     while z:
  ....:         p += 100
  ....:         MRF=NearlySingularMatrix(R(p),n)
  ....:         d = MRF.det()
  ....:         z = d.contains_zero()
  ....:     return p,d
  sage: tryDet(RealBallField,n)  # long time
  (1653, [9.552323592707808e-485 +/- 1.65e-501])

Sage example in ./float.tex, line 2095::

  sage: tryDet(RealIntervalField,n)  # long time
  (1653, 9.552323592707808?e-485)

Sage example in ./float.tex, line 2135::

  sage: CBF(sqrt(2),pi)
  [1.414213562373095...] + [3.141592653589793...]*I
  sage: CIF(sqrt(2),pi)
  1.414213562373095? + 3.141592653589794?*I
  sage: CIF(sqrt(2)+pi*I)
  1.414213562373095? + 3.141592653589794?*I
  sage: CBF(sqrt(2)+pi*I)
  [1.414213562373095...] + [3.141592653589793...]*I

Sage example in ./float.tex, line 2146::

  sage: sage.rings.real_mpfi.printing_style = 'brackets'

Sage example in ./float.tex, line 2149::

  sage: c = CIF(RIF(1,2),RIF(-3,3))
  sage: c.real()
  [1.0000000000000000 .. 2.0000000000000000]
  sage: c.imag()
  [-3.0000000000000000 .. 3.0000000000000000]
  sage: CBF(RIF(1,2),RIF(-3,3))
  [+/- 2.01] + [+/- 3.01]*I

Sage example in ./float.tex, line 2164::

  sage: sage.rings.real_mpfi.printing_style = 'question'

Sage example in ./float.tex, line 2167::

  sage: ComplexIntervalField(100)(1+I*pi).arg()
  1.26262725567891168344432208361?
  sage: ComplexBallField(100)(1+I*pi).arg()
  [1.26262725567891168344432208360 +/- ...e-30]
  sage: ComplexIntervalField(100)(1+I*pi).norm()
  10.8696044010893586188344909999?

Sage example in ./float.tex, line 2242::

  sage: sage.rings.real_mpfi.printing_style = 'question'

Sage example in ./float.tex, line 2245::

  sage: x=QQbar(sqrt(3)); x
  1.732050807568878?
  sage: x.interval(RealIntervalField(100))
  1.73205080756887729352744634151?
"""



"""
This file (./programmation_doctest.sage) was *autogenerated* from ./programmation.tex,
with sagetex.sty version 2011/05/27 v2.3.1.
It contains the contents of all the sageexample environments from this file.
You should be able to doctest this file with:
sage -t ./programmation_doctest.sage
It is always safe to delete this file; it is not used in typesetting your
document.

Sage example in ./programmation.tex, line 118::

  sage: 2*3; 3*4; 4*5          # one comment, 3 results
  6
  12
  20

Sage example in ./programmation.tex, line 137::

  sage: 123 + \
  ....: 345
  468

Sage example in ./programmation.tex, line 201::

  sage: import keyword; sorted(keyword.kwlist)
  [...'and', 'as', 'assert', ..., 'class', 'continue', 'def', 'del'...]

Sage example in ./programmation.tex, line 338::

  sage: y = 3; y = 3 * y + 1; y = 3 * y + 1; y
  31

Sage example in ./programmation.tex, line 368::

  sage: a, b = 10, 20 # (a, b) = (10, 20) and [10, 20] are also possible
  sage: a, b = b, a
  sage: a, b
  (20, 10)

Sage example in ./programmation.tex, line 382::

  sage: temp = a; a = b; b = temp    # equivalent to: a, b = b, a

Sage example in ./programmation.tex, line 391::

  sage: x, y = var('x, y'); a = x ; b = y
  sage: a, b
  (x, y)
  sage: a = a + b ; b = a - b ; a = a - b
  sage: a, b
  (y, x)

Sage example in ./programmation.tex, line 416::

  sage: 2 + 2 == 2^2, 3 * 3 == 3^3
  (True, False)

Sage example in ./programmation.tex, line 483::

  sage: for k in [1..5]:
  ....:    print(7*k)  # block containing a single instruction
  7
  14
  21
  28
  35

Sage example in ./programmation.tex, line 687::

  sage: S = 0 ; k = 0         #        The sum S starts to 0
  sage: while e^k <= 10^6:    #            e^13 <= 10^6 < e^14
  ....:     S = S + k^2       #           accumulates the squares k^2
  ....:     k = k + 1
  sage: S
  819

Sage example in ./programmation.tex, line 734::

  sage: x = 10^4; u = 1; n = 0          # invariant: u = 2^n
  sage: while u <= x: n = n+1; u = 2*u  # or n += 1; u *= 2
  sage: n
  14

Sage example in ./programmation.tex, line 880::

  sage: U = 1.0              # or U = 1. or U = 1.000
  sage: for n in [1..20]:
  ....:   U = 1 / (1 + U^2)
  sage: U
  0.682360434761105

Sage example in ./programmation.tex, line 942::

  sage: S = 0 ; n = 10
  sage: for k in [1..n]:
  ....:     S = S + (2*k) * (2*k+1)
  sage: S
  1650

Sage example in ./programmation.tex, line 961::

  sage: n, k = var('n, k') ; res = sum(2*k*(2*k+1), k, 1, n)
  sage: res, factor(res)    # result expanded, factorised
  (4/3*n^3 + 3*n^2 + 5/3*n, 1/3*(4*n + 5)*(n + 1)*n)

Sage example in ./programmation.tex, line 1074::

  sage: U = 2.0; V = 50.0
  sage: while V-U >= 1.0e-6:      # 1.0e-6 stands for 1.0*10^-6
  ....:   temp = U
  ....:   U = 2 * U * V / (U + V)
  ....:   V = (temp + V) / 2
  sage: U, V
  (9.99999999989256, 10.0000000001074)

Sage example in ./programmation.tex, line 1166::

  sage: U = 0.0        # the sum S0 is empty, of value zero
  sage: V = -1.0       # S1 = -1/1^3
  sage: n = 0          # U and V contain S(2n) and S(2n+1)
  sage: while U-V >= 1.0e-6:
  ....:   n = n+1             # n += 1 is equivalent
  ....:   U = V + 1/(2*n)^3   # going from S(2n-1) to S(2n)
  ....:   V = U - 1/(2*n+1)^3 # going from S(2n) to S(2n+1)
  sage: V, U
  (-0.901543155458595, -0.901542184868447)

Sage example in ./programmation.tex, line 1404::

  sage: u = 6 ; n = 0
  sage: while u != 1:
  ....:   if u % 2 == 0: # the operator % yields the remainder
  ....:     u = u//2     # //: Euclidean division quotient
  ....:   else:
  ....:     u = 3*u+1
  ....:   n = n+1
  sage: n
  8

Sage example in ./programmation.tex, line 1508::

  sage: def fct2 (x, y):
  ....:   return x^2 + y^2
  sage: a = var('a')
  sage: fct2 (a, 2*a)
  5*a^2

Sage example in ./programmation.tex, line 1557::

  sage: def foo (u):
  ....:   t = u^2
  ....:   return t*(t+1)
  sage: t = 1 ; u = 2
  sage: foo(3), t, u
  (90, 1, 2)

Sage example in ./programmation.tex, line 1570::

  sage: a = b = 1
  sage: def f(): global a; a = b = 2
  sage: f(); a, b
  (2, 1)

Sage example in ./programmation.tex, line 1591::

  sage: def AHmean (u, v):
  ....:    u, v = min(u, v), max(u, v)
  ....:    while v-u > 2.0e-8:
  ....:       u, v = 2*u*v/(u+v), (u+v)/2
  ....:    return (u+v) / 2

Sage example in ./programmation.tex, line 1604::

  sage: AHmean (1., 2.)
  1.41421356237309
  sage: AHmean                      # corresponds to a function
    <function AHmean at ...>

Sage example in ./programmation.tex, line 1687::

  sage: def fact1 (n):
  ....:    res = 1
  ....:    for k in [1..n]: res = res*k
  ....:    return res

Sage example in ./programmation.tex, line 1693::

  sage: def fact2 (n):
  ....:    if n == 0: return 1
  ....:    else: return n*fact2(n-1)

Sage example in ./programmation.tex, line 1728::

  sage: def fib1 (n):
  ....:   if n == 0 or n == 1: return n
  ....:   else:
  ....:     U = 0 ; V = 1 # the initial terms u0 and u1
  ....:     for k in [2..n]: W = U+V ; U = V ; V = W
  ....:     return V
  sage: fib1(8)
  21

Sage example in ./programmation.tex, line 1769::

  sage: def fib2 (n):
  ....:   if 0 <= n <= 1: return n      # for n = 0 or n = 1
  ....:   else: return fib2(n-1) + fib2(n-2)

Sage example in ./programmation.tex, line 1857::

  sage: a = 2; n = 6; res = 1         # 1 is the product neutral element
  sage: for k in [1..n]: res = res*a
  sage: res                           # the value of res is 2^6
  64

Sage example in ./programmation.tex, line 1958::

  sage: def pow1 (a, n):
  ....:   if n == 0: return 1
  ....:   elif n % 2 == 0: b = pow1 (a, n//2); return b*b
  ....:   else: return a * pow1(a, n-1)

Sage example in ./programmation.tex, line 1968::

  sage: pow1 (2, 11)                 # result is 2^11
  2048

Sage example in ./programmation.tex, line 2010::

  sage: def pow2 (u, k):
  ....:   v = 1
  ....:   while k != 0:
  ....:     if k % 2 == 0: u = u*u ; k = k//2
  ....:     else: v = v*u ; k = k-1
  ....:   return v

Sage example in ./programmation.tex, line 2022::

  sage: pow2 (2, 10)                 # result is 2^10
  1024

Sage example in ./programmation.tex, line 2109::

  sage: def fib3 (n):
  ....:   A = matrix ([[0, 1], [1, 1]]) ; X0 = vector ([0, 1])
  ....:   return (A^n*X0)[0]

Sage example in ./programmation.tex, line 2114::

  sage: def fib4 (n):
  ....:   return (matrix([[0,1], [1,1]])^n * vector([0,1]))[0]

Sage example in ./programmation.tex, line 2195::

  sage: for k in [1..6]: print('%2d^4 = %4d' % (k, k^4))
   1^4 =    1
   2^4 =   16
   3^4 =   81
   4^4 =  256
   5^4 =  625
   6^4 = 1296

Sage example in ./programmation.tex, line 2274::

  sage: L = [10, 20, 30]
  sage: L
  [10, 20, 30]
  sage: []                    # [] is the empty list
  []

Sage example in ./programmation.tex, line 2299::

  sage: L[1], len(L), len([])
  (20, 3, 0)

Sage example in ./programmation.tex, line 2309::

  sage: L[2] = 33
  sage: L
  [10, 20, 33]

Sage example in ./programmation.tex, line 2318::

  sage: L = [11, 22, 33]
  sage: L[-1], L[-2], L[-3]
  (33, 22, 11)

Sage example in ./programmation.tex, line 2336::

  sage: L = [0, 11, 22, 33, 44, 55]
  sage: L[2:4]
  [22, 33]
  sage: L[-4:4]
  [22, 33]
  sage: L[2:-2]
  [22, 33]
  sage: L[:4]
  [0, 11, 22, 33]
  sage: L[4:]
  [44, 55]

Sage example in ./programmation.tex, line 2359::

  sage: L = [0, 11, 22, 33, 44, 55, 66, 77]
  sage: L[2:6] = [12, 13, 14]        # substitutes [22, 33, 44, 55]

Sage example in ./programmation.tex, line 2393::

  sage: L = [1, 3, 5, 7, 9, 11, 13, 15, 17, 19]
  sage: L[3:len(L)-5] == L[3-len(L):-5]
  True
  sage: [5 in L, 6 in L]
  [True, False]

Sage example in ./programmation.tex, line 2417::

  sage: L = [1, 2, 3] ; L + [10, 20, 30]
  [1, 2, 3, 10, 20, 30]
  sage: 4 * [1, 2, 3]
  [1, 2, 3, 1, 2, 3, 1, 2, 3, 1, 2, 3]

Sage example in ./programmation.tex, line 2441::

  sage: L = 5*[10, 20, 30] ; L[:3]+L[3:] == L
  True

Sage example in ./programmation.tex, line 2459::

  sage: [1..3, 7, 10..13]
  [1, 2, 3, 7, 10, 11, 12, 13]

Sage example in ./programmation.tex, line 2485::

  sage: list(map (cos, [0, pi/6, pi/4, pi/3, pi/2]))
  [1, 1/2*sqrt(3), 1/2*sqrt(2), 1/2, 0]

Sage example in ./programmation.tex, line 2496::

  sage: list(map (lambda t: cos(t), [0, pi/6, pi/4, pi/3, pi/2]))
  [1, 1/2*sqrt(3), 1/2*sqrt(2), 1/2, 0]

Sage example in ./programmation.tex, line 2534::

  sage: list(map (lambda t: N(cos(t)), [0, pi/6, pi/4, pi/3, pi/2]))
  [1.00000000000000, 0.866025403784439, 0.707106781186548,
  0.500000000000000, 0.000000000000000]

Sage example in ./programmation.tex, line 2546::

  sage: list(map (N, map (cos, [0, pi/6, pi/4, pi/3, pi/2])))
  [1.00000000000000, 0.866025403784439, 0.707106781186548,
  0.500000000000000, 0.000000000000000]

Sage example in ./programmation.tex, line 2551::

  sage: list(map (compose(N, cos), [0, pi/6, pi/4, pi/3, pi/2]))
  [1.00000000000000, 0.866025403784439, 0.707106781186548,
  0.500000000000000, 0.000000000000000]

Sage example in ./programmation.tex, line 2564::

  sage: list(filter (is_prime, [1..55]))
  [2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53]

Sage example in ./programmation.tex, line 2578::

  sage: p = 37 ; list(filter (lambda n: n^4 % p == 7, [0..p-1]))
  [3, 18, 19, 34]

Sage example in ./programmation.tex, line 2595::

  sage: list(map(lambda n:2*n+1, [0..15]))
  [1, 3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23, 25, 27, 29, 31]
  sage: [2*n+1 for n in [0..15]]
  [1, 3, 5, 7, 9, 11, 13, 15, 17, 19, 21, 23, 25, 27, 29, 31]

Sage example in ./programmation.tex, line 2607::

  sage: list(filter (is_prime, [1..55]))
  [2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53]
  sage: [p for p in [1..55] if is_prime(p)]
  [2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53]

Sage example in ./programmation.tex, line 2621::

  sage: list(filter (is_prime, [4*n+1 for n in [0..20]]))
  [5, 13, 17, 29, 37, 41, 53, 61, 73]
  sage: [n^2 for n in [1..20] if is_prime(n)]
  [4, 9, 25, 49, 121, 169, 289, 361]

Sage example in ./programmation.tex, line 2651::

  sage: reduce (lambda x, y: 10*x+y, [1, 2, 3, 4])
  1234

Sage example in ./programmation.tex, line 2657::

  sage: reduce (lambda x, y: 10*x+y, [9, 8, 7, 6], 1)
  19876

Sage example in ./programmation.tex, line 2667::

  sage: L = [2*n+1 for n in [0..9]]
  sage: reduce (lambda x, y: x*y, L, 1)
  654729075

Sage example in ./programmation.tex, line 2743::

  sage: prod ([2*n+1 for n in [0..9]], 1) # a list with for
  654729075
  sage: prod ( 2*n+1 for n in [0..9])     # without a list
  654729075
  sage: prod (n for n in [0..19] if n%2 == 1)
  654729075

Sage example in ./programmation.tex, line 2764::

  sage: def fct (x): return 4/x == 2
  sage: all (fct(x) for x in [2, 1, 0])
  False
  sage: any (fct(x) for x in [2, 1, 0])
  True

Sage example in ./programmation.tex, line 2793::

  sage: [[x, y] for x in [1..2] for y in [6..8]]
  [[1, 6], [1, 7], [1, 8], [2, 6], [2, 7], [2, 8]]

Sage example in ./programmation.tex, line 2800::

  sage: [[[x, y] for x in [1..2]] for y in [6..8]]
  [[[1, 6], [2, 6]], [[1, 7], [2, 7]], [[1, 8], [2, 8]]]

Sage example in ./programmation.tex, line 2810::

  sage: list(map (lambda x, y: [x, y], [1..3], [6..8]))
  [[1, 6], [2, 7], [3, 8]]

Sage example in ./programmation.tex, line 2829::

  sage: L = [[1, 2, [3]], [4, [5, 6]], [7, [8, [9]]]]
  sage: flatten (L, max_level = 1)
  [1, 2, [3], 4, [5, 6], 7, [8, [9]]]
  sage: flatten (L, max_level = 2)
  [1, 2, 3, 4, 5, 6, 7, 8, [9]]
  sage: flatten (L)             # equivalent to flatten (L, max_level = 3)
  [1, 2, 3, 4, 5, 6, 7, 8, 9]

Sage example in ./programmation.tex, line 2852::

  sage: x = var('x')
  sage: factor(diff(x*exp(x), [x, x]))
  (x + 2)*e^x
  sage: list(map(lambda n: factor(diff(x*exp(x), n*[x])), [0..6]))
  [x*e^x, (x + 1)*e^x, (x + 2)*e^x, (x + 3)*e^x, (x + 4)*e^x,
  (x + 5)*e^x, (x + 6)*e^x]
  sage: [factor (diff (x*exp(x), n*[x])) for n in [0..6]]
  [x*e^x, (x + 1)*e^x, (x + 2)*e^x, (x + 3)*e^x, (x + 4)*e^x,
  (x + 5)*e^x, (x + 6)*e^x]

Sage example in ./programmation.tex, line 2907::

  sage: L = [1, 8, 5, 2, 9] ; L.reverse() ; L
  [9, 2, 5, 8, 1]
  sage: L.sort() ; L
  [1, 2, 5, 8, 9]
  sage: L.sort(reverse = True) ; L
  [9, 8, 5, 2, 1]

Sage example in ./programmation.tex, line 3031::

  sage: def homogLex (P, Q):
  ....:  sp = sum (P) ; sq = sum (Q)
  ....:  if sp < sq: return int(-1)
  ....:  elif sp > sq: return int(1)
  ....:  else: return alpha (P, Q)

Sage example in ./programmation.tex, line 3038::

  sage: homogLex ([2, 3, 4, 6, 4], [2, 3, 4, 5, 6])
  -1

Sage example in ./programmation.tex, line 3121::

  sage: def fct1(L):
  ....:   return [list(filter(lambda n: n % 2 == 0, L)),
  ....:           list(filter(lambda n: n % 2 == 1, L))]

Sage example in ./programmation.tex, line 3126::

  sage: fct1([1..10])
  [[2, 4, 6, 8, 10], [1, 3, 5, 7, 9]]

Sage example in ./programmation.tex, line 3145::

  sage: def fct2 (L):
  ....:   res0 = [] ; res1 = []
  ....:   for k in L:
  ....:     if k%2 == 0: res0.append(k) # or res0[len(res0):] = [k]
  ....:     else: res1.append(k)        # or res1[len(res1):] = [k]
  ....:   return [res0, res1]

Sage example in ./programmation.tex, line 3157::

  sage: def fct3a (L, res0, res1):
  ....:   if L == []: return [res0, res1]
  ....:   elif L[0]%2 == 0: return fct3a(L[1:], res0+[L[0]], res1)
  ....:   else: return fct3a (L[1:], res0, res1+[L[0]])

Sage example in ./programmation.tex, line 3163::

  sage: def fct3 (L): return fct3a (L, [], [])

Sage example in ./programmation.tex, line 3195::

  sage: def subSequences (L):
  ....:   if L == []: return []
  ....:   res = [] ; start = 0 ; k = 1
  ....:   while k < len(L):    # 2 consecutive terms are defined
  ....:     if L[k-1] > L[k]:
  ....:       res.append (L[start:k]) ; start = k
  ....:     k = k+1
  ....:   res.append (L[start:k])
  ....:   return res

Sage example in ./programmation.tex, line 3212::

  sage: subSequences([1, 4, 1, 5])
  [[1, 4], [1, 5]]
  sage: subSequences([4, 1, 5, 1])
  [[4], [1, 5], [1]]

Sage example in ./programmation.tex, line 3255::

  sage: S = 'This is a character string.'

Sage example in ./programmation.tex, line 3277::

  sage: S = 'This is a déjà-vu example.'
  sage: print(S)
  This is a déjà-vu example.

Sage example in ./programmation.tex, line 3322::

  sage: S='one two three four five six seven'; L=S.split(); L
  ['one', 'two', 'three', 'four', 'five', 'six', 'seven']

Sage example in ./programmation.tex, line 3368::

  sage: L1 = [11, 22, 33] ; L2 = L1
  sage: L1[1] = 222 ; L2.sort() ; L1, L2
  ([11, 33, 222], [11, 33, 222])
  sage: L1[2:3] = []; L2[0:0] = [6, 7, 8]
  sage: L1, L2
  ([6, 7, 8, 11, 33], [6, 7, 8, 11, 33])

Sage example in ./programmation.tex, line 3422::

  sage: L1 = [11, 22, 33] ; L2 = L1 ; L3 = L1[:]
  sage: [L1 is L2, L2 is L1, L1 is L3, L1 == L3]
  [True, True, False, True]

Sage example in ./programmation.tex, line 3439::

  sage: La = [1, 2, 3] ; L1 = [1, La] ; L2 = copy(L1)
  sage: L1[1][0] = 5         # [1, [5, 2, 3]] for L1 and L2
  sage: [L1 == L2, L1 is L2, L1[1] is L2[1]]
  [True, False, True]

Sage example in ./programmation.tex, line 3498::

  sage: def lexInverse (P, Q):
  ....:   P1 = copy(P) ; P1.reverse()
  ....:   Q1 = copy(Q) ; Q1.reverse()
  ....:   return - alpha (P1, Q1)

Sage example in ./programmation.tex, line 3593::

  sage: S0 = (); S1 = (1, ); S2 = (1, 2)
  sage: [1 in S1, 1 == (1)]
  [True, True]

Sage example in ./programmation.tex, line 3610::

  sage: S1 = (1, 4, 9, 16, 25); [k for k in S1]
  [1, 4, 9, 16, 25]

Sage example in ./programmation.tex, line 3627::

  sage: L1 = [0..4]; L2 = [5..9]
  sage: list(zip(L1, L2))
  [(0, 5), (1, 6), (2, 7), (3, 8), (4, 9)]
  sage: list(map(lambda x, y:(x, y), L1, L2))
  [(0, 5), (1, 6), (2, 7), (3, 8), (4, 9)]

Sage example in ./programmation.tex, line 3656::

  sage: E = Set([1, 2, 4, 8, 2, 2, 2]); F = Set([7, 5, 3, 1]); E, F
  ({8, 1, 2, 4}, {1, 3, 5, 7})

Sage example in ./programmation.tex, line 3678::

  sage: E = Set([1, 2, 4, 8, 2, 2, 2]); F = Set([7, 5, 3, 1])
  sage: 5 in E, 5 in F, E + F == F | E
  (False, True, True)
  sage: E & F, E - F, E ^^ F
  ({1}, {8, 2, 4}, {2, 3, 4, 5, 7, 8})

Sage example in ./programmation.tex, line 3700::

  sage: E = Set([1, 2, 4, 8, 2, 2, 2])
  sage: [E[k] for k in [0..len(E)-1]], [t for t in E]
  ([8, 1, 2, 4], [8, 1, 2, 4])

Sage example in ./programmation.tex, line 3713::

  sage: def included (E, F): return E+F == F

Sage example in ./programmation.tex, line 3728::

  sage: sorted(Set([Set([]), Set([1]), Set([2]), Set([1, 2])]), key=str)
  [{1, 2}, {1}, {2}, {}]
  sage: sorted(Set([    (),     (1, ),    (2, ),    (1, 2) ]))
  [(), (1,), (1, 2), (2,)]

Sage example in ./programmation.tex, line 3744::

  sage: def Parts (EE):
  ....:   if EE == Set([]): return Set([EE])
  ....:   else:
  ....:     return withOrWithout (EE[0], Parts(Set(EE[1:])))

Sage example in ./programmation.tex, line 3754::

  sage: def withOrWithout (a, E):
  ....:   return Set (map (lambda F: Set([a])+F, E)) + E

Sage example in ./programmation.tex, line 3762::

  sage: sorted(Parts(Set([1, 2, 3])), key=str)
  [{1, 2, 3}, {1, 2}, {1, 3}, {1}, {2, 3}, {2}, {3}, {}]

Sage example in ./programmation.tex, line 3804::

  sage: D={}; D['one']=1; D['two']=2; D['three']=3; D['ten']=10
  sage: D['two'] + D['three']
  5

Sage example in ./programmation.tex, line 3857::

  sage: D = {'a0':'b0', 'a1':'b1', 'a2':'b2', 'a3':'b0',\
  ....: 'a4':'b3', 'a5':'b3'}
  sage: E  = Set(D.keys()) ; Imf = Set(D.values())
  sage: Imf == Set(map (lambda t:D[t], E))     # is equivalent
  True

Sage example in ./programmation.tex, line 3894::

  sage: def injective(D):
  ....:   return len(D) == len (Set(D.values()))
"""



"""
This file (./integration_doctest.sage) was *autogenerated* from ./integration.tex,
with sagetex.sty version 2011/05/27 v2.3.1.
It contains the contents of all the sageexample environments from this file.
You should be able to doctest this file with:
sage -t ./integration_doctest.sage
It is always safe to delete this file; it is not used in typesetting your
document.

Sage example in ./integration.tex, line 73::

  sage: x = var('x'); f(x) = exp(-x^2) * log(x)
  sage: result = integrate(f, x, 1, 3)
  ...
  sage: N(result)  # abs tol 1e-14
  0.03586029499126769

Sage example in ./integration.tex, line 78::

  sage: plot(f, 1, 3, fill='axis')
  Graphics object consisting of 2 graphics primitives

Sage example in ./integration.tex, line 104::

  sage: N(integrate(sin(x^2)/(x^2), x, 1, infinity))  # abs tol 2e-15
  0.285736646322853 - 6.93889390390723e-18*I

Sage example in ./integration.tex, line 108::

  sage: plot(sin(x^2)/(x^2), x, 1, 10, fill='axis')
  Graphics object consisting of 2 graphics primitives

Sage example in ./integration.tex, line 162::

  sage: fp = plot(f, 1, 3, color='red')
  sage: n = 4
  sage: interp_points = [(1+2*u/(n-1), N(f(1+2*u/(n-1))))
  ....:                  for u in range(n)]
  sage: A = PolynomialRing(RR, 'x')
  sage: pp = plot(A.lagrange_polynomial(interp_points), 1, 3, fill='axis')
  sage: fp+pp
  Graphics object consisting of 3 graphics primitives

Sage example in ./integration.tex, line 522::

  sage: N(integrate(cos(log(cos(x))), x, 0, pi/4))  # rel tol 2e-12
  0.7766520331543109

Sage example in ./integration.tex, line 536::

  sage: integrate(log(1+x)*x, x, 0, 1)
  1/4
  sage: N(integrate(log(1+x)*x, x, 0, 1))
  0.250000000000000

Sage example in ./integration.tex, line 562::

  sage: numerical_integral(cos(log(cos(x))), 0, pi/4)  # rel tol 2e-11
  (0.7766520331543109, 8.622569693298564e-15)

Sage example in ./integration.tex, line 600::

  sage: numerical_integral(exp(-x^100), 0, 1.1)
  (0.99432585119150..., 4.0775730...e-09)
  sage: numerical_integral(exp(-x^100), 0, 1.1, algorithm='qng') # abs tol 2e-12
  (0.9943275385765319, 0.016840666914705607)

Sage example in ./integration.tex, line 612::

  sage: integrate(cos(log(cos(x))), x, 0, pi/4)
  integrate(cos(log(cos(x))), x, 0, 1/4*pi)

Sage example in ./integration.tex, line 622::

  sage: N(integrate(cos(log(cos(x))), x, 0, pi/4), digits=60) # abs tol 2e-12
  0.7766520331543109

Sage example in ./integration.tex, line 628::

  sage: N(integrate(sin(x)*exp(cos(x)), x, 0, pi), digits=60)
  2.35040238728760291376476370119120163031143596266819174045913

Sage example in ./integration.tex, line 644::

  sage: sage.calculus.calculus.nintegral(sin(sin(x)), x, 0, 1)
  (0.430606103120690..., 4.78068810228705...e-15, 21, 0)

Sage example in ./integration.tex, line 654::

  sage: g = sin(sin(x))
  sage: g.nintegral(x, 0, 1)
  (0.430606103120690..., 4.78068810228705...e-15, 21, 0)

Ensure consistent results on 32-bit and 64-bit systems by using the same
precision::

  sage: _ = gp.default('realprecision', 38)

Sage example in ./integration.tex, line 703::

  sage: gp('intnum(x=17, 20, exp(-x^2)*log(x))')
  2.5657285005610514829173563961304785900 E-127

Sage example in ./integration.tex, line 717::

  sage: gp('intnum(x=0, 1, sin(sin(x)))')
  0.43060610312069060491237735524846578643
  sage: old_prec = gp.set_precision(50)
  sage: gp('intnum(x=0, 1, sin(sin(x)))')
  0.43060610312069060491237735524846578643360804182200

Sage example in ./integration.tex, line 746::

  sage: p = gp.set_precision(old_prec) # we reset the default precision
  sage: gp('intnum(x=0, 1, x^(-99/100))')  # rel tol 1e-9
  73.629142577870966597465391764897770039

Sage example in ./integration.tex, line 754::

  sage: gp('intnum(x=[0, -99/100], 1, x^(-99/100))')
  100.00000000000000000000000000000000000

Sage example in ./integration.tex, line 766::

  sage: gp('intnum(x=[0, -1/42], 1, x^(-99/100))')  # rel tol 1e-9
  74.472749314025559405335761513474670714

Sage example in ./integration.tex, line 785::

  sage: import mpmath
  sage: mpmath.mp.prec = 53
  sage: mpmath.quad(lambda x: mpmath.sin(mpmath.sin(x)), [0, 1])
  mpf('0.43060610312069059')

Sage example in ./integration.tex, line 795::

  sage: a = RDF(pi); b = mpmath.mpf(a); b
  mpf('3.1415926535897931')
  sage: c = RDF(b); c
  3.141592653589793

Sage example in ./integration.tex, line 824::

  sage: mpmath.mp.prec = 113
  sage: mpmath.quad(lambda x: mpmath.sin(mpmath.sin(x)), [0, 1])
  mpf('0.430606103120690604912377355248465809')

Sage example in ./integration.tex, line 846::

  sage: f(x) = sin(sin(x))
  sage: mpmath.quad(f, [0, 1])
  Traceback (most recent call last):
  ...
  TypeError: no canonical coercion from <class 'sage.libs.mpmath.ext_main.mpf'> to ...

Sage example in ./integration.tex, line 866::

  sage: g(x) = max_symbolic(sin(x), cos(x))
  sage: mpmath.mp.prec = 100
  sage: mpmath.quadts(lambda x: g(N(x, 100)), [0, 1])
  mpf('0.873912416263035435957979086252')

Sage example in ./integration.tex, line 878::

  sage: mpmath.mp.prec = 170
  sage: mpmath.quadts(lambda x: g(N(x, 190)), [0, 1])
  mpf('0.87391090757400975205393005981962476344054148354188794')
  sage: N(sqrt(2) - cos(1), 100)
  0.87391125650495533140075211677

Sage example in ./integration.tex, line 892::

  sage: mpmath.quadts(lambda x: g(N(x, 170)), [0, mpmath.pi / 4, 1])
  mpf('0.87391125650495533140075211676672147483736145475902551')

Sage example in ./integration.tex, line 979::

  sage: y = var('y'); integrate(exp(y*sin(x)), (x, 0, sqrt(y)))  # long time
  integrate(e^(y*sin(x)), x, 0, sqrt(y))

Sage example in ./integration.tex, line 990::

  sage: f = lambda y: numerical_integral(lambda x: exp(y*sin(x)),  \
                                         0, sqrt(y))[0]
  sage: f(0.0), f(0.5), f(1.0) # abs tol 2e-15
  (0.0, 0.8414895067661431, 1.6318696084180513)

Sage example in ./integration.tex, line 998::

  sage: numerical_integral(f, 0, 1) # abs tol 2e-16
  (0.8606791942204567, 6.301207560882073e-07)

Sage example in ./integration.tex, line 1008::

  sage: f = lambda y: sage.calculus.calculus.nintegral(exp(y*sin(x)), \
                                                       x, 0, sqrt(y))[0]
  sage: numerical_integral(f, 0, 1) # abs tol 2e-16
  (0.8606791942204567, 6.301207560882096e-07)

Sage example in ./integration.tex, line 1016::

  sage: f = lambda y: RDF(mpmath.quad(lambda x: mpmath.exp(y*mpmath.sin(x)), \
                                      [0, sqrt(y)]))
  sage: numerical_integral(f, 0, 1) # abs tol 2e-16
  (0.8606791942204567, 6.301207561187562e-07)

Sage example in ./integration.tex, line 1027::

  sage: mpmath.mp.dps = 60
  sage: f = lambda x, y: mpmath.exp(y*mpmath.sin(x))
  sage: mpmath.quad(f, [0,1], [0,1])
  mpf('1.28392205755238471754385917646324675741664250325189751108716305')

Sage example in ./integration.tex, line 1044::

  sage: def evalI(n):
  ....:   f = lambda y: numerical_integral(lambda x: exp(y*sin(x)),
  ....:               0, sqrt(y), algorithm='qng', max_points=n)[0]
  ....:   return numerical_integral(f, 0, 1, algorithm='qng', max_points=n)
  sage: evalI(100) # abs tol 2e-12
  (0.8606792028826138, 5.553962923506737e-07)

Sage example in ./integration.tex, line 1228::

  sage: T = ode_solver()

Sage example in ./integration.tex, line 1244::

  sage: def f_1(t,y,params): return [y[1],params[0]*(1-y[0]^2)*y[1]-y[0]]
  sage: T.function = f_1

Sage example in ./integration.tex, line 1266::

  sage: def j_1(t,y,params):
  ....:     return [[0, 1],
  ....:             [-2*params[0]*y[0]*y[1]-1, params[0]*(1-y[0]^2)],
  ....:             [0,0]]
  sage: T.jacobian = j_1

Sage example in ./integration.tex, line 1279::

  sage: T.algorithm = "rk8pd"
  sage: T.ode_solve(y_0=[1,0], t_span=[0,100], params=[10],
  ....:             num_points=1000)
  sage: f = T.interpolate_solution()

Sage example in ./integration.tex, line 1302::

  sage: plot(f, 0, 100)
  Graphics object consisting of 1 graphics primitive

Sage example in ./integration.tex, line 1363::

  sage: t, y = var('t, y')
  sage: desolve_rk4(t*y*(2-y), y, ics=[0,1], end_points=[0, 1], step=0.5)  # abs tol 1e-12
  [[0, 1], [0.5, 1.12419127424558], [1.0, 1.461590162288825]]

Sage example in ./integration.tex, line 1399::

  sage: import mpmath
  sage: mpmath.mp.prec = 53
  sage: sol = mpmath.odefun(lambda t, y: y, 0, 1)
  sage: sol(1)
  mpf('2.7182818284590451')
  sage: mpmath.mp.prec = 100
  sage: sol(1)
  mpf('2.7182818284590452353602874802307')
  sage: N(exp(1), 100)
  2.7182818284590452353602874714

Sage example in ./integration.tex, line 1436::

  sage: mpmath.mp.prec = 53
  sage: f = mpmath.odefun(lambda t, y: [-y[1], y[0]], 0, [1, 0])
  sage: f(3)
  [mpf('-0.98999249660044542'), mpf('0.14112000805986721')]
  sage: (cos(3.), sin(3.))
  (-0.989992496600445, 0.141120008059867)

Sage example in ./integration.tex, line 1497::

  sage: mpmath.mp.prec = 10
  sage: sol = mpmath.odefun(lambda t, y: y, 0, 1)
  sage: sol(1)
  mpf('2.7148')
  sage: mpmath.mp.prec = 100
  sage: sol(1)
  mpf('2.7135204235459511323824699502438')
"""



"""
This file (./graphtheory_doctest.sage) was *autogenerated* from ./graphtheory.tex,
with sagetex.sty version 2011/05/27 v2.3.1.
It contains the contents of all the sageexample environments from this file.
You should be able to doctest this file with:
sage -t ./graphtheory_doctest.sage
It is always safe to delete this file; it is not used in typesetting your
document.

Sage example in ./graphtheory.tex, line 12::

  sage: set_random_seed(1588)
  sage: sage.numerical.backends.generic_backend.default_solver = "Glpk"

Sage example in ./graphtheory.tex, line 59::

  sage: g = Graph()

Sage example in ./graphtheory.tex, line 87::

  sage: g.order(), g.size()
  (0, 0)
  sage: g.add_vertex(0)
  sage: g.order(), g.size()
  (1, 0)
  sage: g.add_vertices([1, 2, 5, 9])
  sage: g.order(), g.size()
  (5, 0)
  sage: g.add_edges([(1,5), (9,2), (2,5), (1,9)])
  sage: g.order(), g.size()
  (5, 4)
  sage: g.add_edge("Madrid", "Edinburgh")
  sage: g.order(), g.size()
  (7, 5)

Sage example in ./graphtheory.tex, line 128::

  sage: g.delete_vertex(0)
  sage: g.delete_edges([(1,5), (2,5)])
  sage: g.order(), g.size()
  (6, 3)
  sage: sorted(g.vertices(sort=False), key=str)
  [1, 2, 5, 9, 'Edinburgh', 'Madrid']
  sage: sorted(g.edges(sort=False, labels=False), key=str)
  [('Edinburgh', 'Madrid'), (1, 9), (2, 9)]

Sage example in ./graphtheory.tex, line 159::

  sage: g = Graph({
  ....:      0: [],
  ....:      1: [5, 9],
  ....:      2: [1, 5, 9],
  ....:      'Edinburgh': ['Madrid']})

Sage example in ./graphtheory.tex, line 212::

  sage: P = graphs.PetersenGraph()
  sage: C = graphs.ChvatalGraph()

Sage example in ./graphtheory.tex, line 252::

  sage: P = graphs.PetersenGraph()
  sage: P.is_planar()
  False
  sage: P.minor(graphs.CompleteBipartiteGraph(3,3))  # random
  {0: [1], 1: [8], 2: [4], 3: [6, 7, 9], 4: [2, 3], 5: [0, 5]}
  sage: P.minor(graphs.CompleteGraph(5))  # random
  {0: [1, 6], 1: [0, 5], 2: [2, 7], 3: [4, 9], 4: [3, 8]}
  sage: P.girth()
  5
  sage: P.is_regular(3)
  True
  sage: P.chromatic_number()
  3
  sage: P.is_vertex_transitive()
  True
  sage: P.show()

Sage example in ./graphtheory.tex, line 332::

  sage: K = graphs.KneserGraph(5, 2); P = graphs.PetersenGraph()
  sage: K.is_isomorphic(P)
  True

Sage example in ./graphtheory.tex, line 340::

  sage: all( graphs.KneserGraph(n,k).chromatic_number() == n - 2*k + 2
  ....:      for n in range(5, 9) for k in range(2, n // 2) )
  True

Sage example in ./graphtheory.tex, line 459::

  sage: H = graphs.ClawGraph()
  sage: def test():
  ....:    g = graphs.RandomGNP(20,2/5)
  ....:    return not g.subgraph_search(H, induced=True) is None
  sage: sum( test() for i in range(100) ) >= 80
  True

Sage example in ./graphtheory.tex, line 485::

  sage: P = graphs.PetersenGraph()
  sage: H = graphs.HoffmanSingletonGraph()
  sage: U = P + H; U2 = P.disjoint_union(H)
  sage: U.is_isomorphic(U2)
  True

Sage example in ./graphtheory.tex, line 496::

  sage: C = graphs.ChvatalGraph()
  sage: U = 3 * C; U2 = C.disjoint_union(C.disjoint_union(C))
  sage: U2.is_isomorphic(U)
  True

Sage example in ./graphtheory.tex, line 506::

  sage: U = 3*P + 2*C

Sage example in ./graphtheory.tex, line 513::

  sage: all( (CC.is_isomorphic(P) or CC.is_isomorphic(C))
  ....:       for CC in U.connected_components_subgraphs() )
  True

Sage example in ./graphtheory.tex, line 522::

  sage: sum( CC.is_isomorphic(P)
  ....:      for CC in U.connected_components_subgraphs() )
  3
  sage: sum( CC.is_isomorphic(C)
  ....:      for CC in U.connected_components_subgraphs() )
  2

Sage example in ./graphtheory.tex, line 568::

  sage: C = graphs.ChvatalGraph(); C.show()

Sage example in ./graphtheory.tex, line 583::

  sage: C.show(partition = [C.independent_set()])

Sage example in ./graphtheory.tex, line 597::

  sage: C.show(vertex_colors = {
  ....:   "red" : [0, 1, 2],    "blue" : [3, 4, 5],
  ....:   "yellow" : [6, 7, 8], "purple" : [9, 10, 11]})

Sage example in ./graphtheory.tex, line 608::

  sage: C.coloring(hex_colors = True)
  {'#00ffff': [3, 8, 5],
     '#7f00ff': [11],
     '#7fff00': [1, 4, 6, 9],
     '#ff0000': [0, 2, 7, 10]}
  sage: C.show(vertex_colors = C.coloring(hex_colors = True))

Sage example in ./graphtheory.tex, line 644::

  sage: L = [graphs.CompleteGraph(i) for i in range(3,3+10)]
  sage: for number, G in enumerate(L):
  ....:     G.plot().save(tmp_filename(ext='.png'))

Sage example in ./graphtheory.tex, line 782::

  sage: P5 = graphs.PathGraph(5); House = graphs.HouseGraph()
  sage: P5.complement().is_isomorphic(House)
  True
  sage: P4 = graphs.PathGraph(4); P4.complement().is_isomorphic(P4)
  True
  sage: C5 = graphs.CycleGraph(5); C5.complement().is_isomorphic(C5)
  True

Sage example in ./graphtheory.tex, line 852::

  sage: n = 5; Path = graphs.PathGraph(n)
  sage: Grid = Path.cartesian_product(Path)
  sage: Grid.is_isomorphic(graphs.GridGraph([n,n]))
  True

Sage example in ./graphtheory.tex, line 1144::

  sage: n = 30; p = 0.3; trials = 50
  sage: def equality(G):
  ....:     return G.edge_connectivity() == min(G.degree())
  sage: sum(equality(graphs.RandomGNP(n,p)) for i in range(trials))/trials
  1

Sage example in ./graphtheory.tex, line 1319::

  sage: g = graphs.ChvatalGraph(); cycle = g.hamiltonian_cycle()
  sage: g.show(vertex_labels = False); cycle.show(vertex_labels = False)

Sage example in ./graphtheory.tex, line 1586::

  sage: set_random_seed(3291)

Sage example in ./graphtheory.tex, line 1589::

  sage: n = 100; p = 5/n; g = graphs.RandomGNP(n, p)

Sage example in ./graphtheory.tex, line 1597::

  sage: # Set of available colors.
  sage: # In the worst-case scenario up to n colors suffice
  sage: available_colors = Set(range(n))

Sage example in ./graphtheory.tex, line 1611::

  sage: # This dictionary contains the color associated
  sage: # with each vertex of the graph
  sage: color = {}
  sage: for u in g:
  ....:    forbidden = Set([color[v] for v in g.neighbors(u)
  ....:                              if v in color])
  ....:    color[u] = min(available_colors - forbidden)

Sage example in ./graphtheory.tex, line 1625::

  sage: # Number of colors used
  sage: max(color.values()) + 1
  5

Sage example in ./graphtheory.tex, line 1635::

  sage: P = Permutations([0,1,2,3]); P.random_element()
  [3, 2, 1, 0]

Sage example in ./graphtheory.tex, line 1646::

  sage: available_colors = Set(range(n))

Sage example in ./graphtheory.tex, line 1656::

  sage: n_tests = 30
  sage: vertices = g.vertices(sort=False)
  sage: P = Permutations(range(n))
  sage: best_coloring = {}
  sage: best_chromatic_number = +oo

Sage example in ./graphtheory.tex, line 1678::

  sage: for t in range(n_tests):
  ....:    # Random ordering of vertices
  ....:    p = P.random_element()
  ....:    color = {}
  ....:    for i in range(g.order()):
  ....:          u = vertices[p[i]]
  ....:          forbidden = Set([color[v] for v in g.neighbors(u)
  ....:                      if v in color])
  ....:          color[u] = min(available_colors - forbidden)
  ....:    # Update the best coloring
  ....:    if max(color.values()) + 1 < best_chromatic_number:
  ....:          best_coloring = color
  ....:          best_chromatic_number = 1 + max(color.values())

Sage example in ./graphtheory.tex, line 1697::

  sage: best_chromatic_number # Number of colors used
  5

Sage example in ./graphtheory.tex, line 1718::

  sage: def greedy_coloring(g, permutation):
  ....:     n = g.order()
  ....:     available_colors = Set(range(n))
  ....:     vertices = g.vertices(sort=False)
  ....:     color = {}
  ....:     for i in range(n):
  ....:         u = vertices[permutation[i]]
  ....:         forbidden = Set([color[v] for v in g.neighbors(u)
  ....:                         if v in color])
  ....:         color[u] = min(available_colors - forbidden)
  ....:     return max(color.values()) + 1, color

Sage example in ./graphtheory.tex, line 1736::

  sage: set_random_seed(0)

Sage example in ./graphtheory.tex, line 1746::

  sage: P = Permutations(range(g.order()))
  sage: n_colors, coloration = min([greedy_coloring(g, P.random_element())
  ....:     for i in range(50)], key=lambda c: c[0])
  sage: n_colors
  5

Sage example in ./graphtheory.tex, line 1782::

  sage: n = 20; k = 4; g = graphs.RandomGNP(n, 0.5)
  sage: g = g.subgraph(edges = g.min_spanning_tree())

Sage example in ./graphtheory.tex, line 1786::

  sage: while True:
  ....:    _, edges, [S,Sb] = g.edge_connectivity(vertices = True)
  ....:    cardinality = len(edges)
  ....:    if cardinality < k:
  ....:         CP = cartesian_product([S, Sb])
  ....:         g.add_edges([CP.random_element()
  ....:                      for i in range(k - len(edges))])
  ....:    else:
  ....:         break

Sage example in ./graphtheory.tex, line 1844::

  sage: g = graphs.RandomGNP(40, 0.4)
  sage: P = Permutations(range(g.order()))
  sage: mean = sum( 1/(g.degree(v)+1) for v in g )

Sage example in ./graphtheory.tex, line 1849::

  sage: while True:
  ....:    n = P.random_element()
  ....:    S = [v for v in g if all( n[v] < n[u] for u in g.neighbors(v))]
  ....:    if len(S) >= mean:
  ....:        break

Sage example in ./graphtheory.tex, line 1989::

  sage: def find_induced(H, G):
  ....:  # the function from V(H) to V(G) we aim to define:
  ....:  f = {}
  ....:  # set of vertices of G not yet used by f:
  ....:  G_remain = G.vertices(sort=False)
  ....:  # set of vertices having no representative yet:
  ....:  H_remain = H.vertices(sort=False)
  ....:  # while the function is not complete:
  ....:  while H_remain:
  ....:      v = H_remain.pop(0) # look for the next vertex of H
  ....:      # and its potential images in G
  ....:      candidates = [u for u in G_remain
  ....:                    if all(H.has_edge(h,v) == G.has_edge(f_h,u)
  ....:                           for h, f_h in f.items())]
  ....:      # if no candidate is found, we abort immediately
  ....:      if not candidates:
  ....:          raise ValueError("No copy of H has been found in G")
  ....:      # otherwise we select the first candidate
  ....:      f[v] = candidates[0]
  ....:      G_remain.remove(f[v])
  ....:  return f

Sage example in ./graphtheory.tex, line 2012::

  sage: set_random_seed(4)

Sage example in ./graphtheory.tex, line 2021::

  sage: H = graphs.PetersenGraph()
  sage: G = graphs.RandomGNP(500,0.5)
  sage: find_induced(H,G)
  {0: 0, 1: 1, 2: 3, 3: 13, 4: 7, 5: 62, 6: 24, 7: 232, 8: 67, 9: 45}

Sage example in ./graphtheory.tex, line 2070::

  sage: n = 100; V = range(n+1)
  sage: G = Graph()
  sage: G.add_edges([
  ....:    (i,j) for i,j in Subsets(V,2) if is_square(abs(i-j)) ])

Sage example in ./graphtheory.tex, line 2084::

  sage: X = G.independent_set(); X  # random with python3
  [4, 6, 9, 11, 16, 21, 23, 26, 28, 33, 38, 43, 50,
  56, 61, 71, 76, 78, 83, 88, 93, 95, 98, 100]
  sage: G.is_independent_set(X)
  True

Sage example in ./graphtheory.tex, line 2139::

  sage: tasks = {0: [2, 5, 3, 7], 1: [0, 1, 4],
  ....:   2: [5, 0, 4],           3: [0, 1],
  ....:   4: [8],                 5: [2],
  ....:   6: [8, 9, 7],           7: [5, 8, 7],
  ....:   8: [2, 5, 3, 6, 4],     9: [2, 5, 8, 6, 1]}
  sage: G = Graph()
  sage: for i in tasks:
  ....:   G.add_edges(("w" + str(i), "t" + str(j)) for j in tasks[i])

Sage example in ./graphtheory.tex, line 2182::

  sage: M = Graph(G.matching())
  sage: txt = "t{} assigned to {}"
  sage: for i in tasks:                                        # random
  ....:     print(txt.format(i, M.neighbors('t' + str(i))[0])) # random
  t0 assigned to w2
  t1 assigned to w3
  t2 assigned to w5
  t3 assigned to w8
  t4 assigned to w1
  t5 assigned to w7
  t6 assigned to w9
  t7 assigned to w0
  t8 assigned to w4
  t9 assigned to w6

Sage example in ./graphtheory.tex, line 2232::

  sage: n = 10
  sage: G = graphs.CompleteGraph(n)
  sage: from sage.graphs.graph_coloring import edge_coloring
  sage: for day, matches in enumerate(edge_coloring(G)):
  ....:    print("Matches of day {}: {}".format(day, matches))
  Matches of day 0: [(0, 9), (1, 8), (2, 7), (3, 6), (4, 5)]
  Matches of day 1: [(0, 2), (1, 9), (3, 8), (4, 7), (5, 6)]
  Matches of day 2: [(0, 4), (1, 3), (2, 9), (5, 8), (6, 7)]
  Matches of day 3: [(0, 6), (1, 5), (2, 4), (3, 9), (7, 8)]
  Matches of day 4: [(0, 8), (1, 7), (2, 6), (3, 5), (4, 9)]
  Matches of day 5: [(0, 1), (2, 8), (3, 7), (4, 6), (5, 9)]
  Matches of day 6: [(0, 3), (1, 2), (4, 8), (5, 7), (6, 9)]
  Matches of day 7: [(0, 5), (1, 4), (2, 3), (6, 8), (7, 9)]
  Matches of day 8: [(0, 7), (1, 6), (2, 5), (3, 4), (8, 9)]

Sage example in ./graphtheory.tex, line 2260::

  sage: g = graphs.CompleteGraph(10)
  sage: g.show(edge_colors=edge_coloring(g, hex_colors=True))
"""
